function initializeConsole() {
	var method;
	var noop = function() {
	};
	var methods = [ 'info', 'log', 'time', 'timeEnd', 'debug', 'trace', 'warn' ];
	var length = methods.length;
	var console = (window.console = window.console || {});

	while (length--) {
		method = methods[length];

		// Only stub undefined methods.
		if (!console[method]) {
			console[method] = noop;
		}
	}
};

var HeatingControl;

function tabActivation(bsettingsMissing) {
	if (bsettingsMissing) {
		HeatingControl.tabControl.setActiveTab(HeatingControl.generalTab);
		HeatingControl.startTab.tab.setDisabled(false);
		HeatingControl.temperatureTab.tab.setDisabled(true);
		HeatingControl.timeTab.tab.setDisabled(true);
	} else {
		HeatingControl.startTab.tab.setDisabled(false);
		HeatingControl.temperatureTab.tab.setDisabled(false);
		HeatingControl.timeTab.tab.setDisabled(false);
	}
    HeatingControl.loadStatusPanel();
}

var OperationModes = {
	Off : 0,
	Comfort : 1,
	Eco : 2,
	NotUsed : 3,
	Night : 4,
	Holiday : 5,
	nothing : -1,
	transparent : -2
};

var ControlModes = {
	cControlModeOff : 0,
	cControlModePID : 1,
	cControlModeZoneFollower : 2,
	cControlModeFixedValue : 3,
	cControlModeManualValue : 4,
	cControlModeAutomatic : 5
};

var AutomaticModeError = {
    AutoOff : 0,
    AutoUnknown : -1,
    AutoNothingSelected : -4,   //no sensor in zone
	AutoSelectionFailed : -5,   //inconsistent valve types
	AutoSelectionNotPossible : -6   //all valve types unknown
};

Ext.define('DSS.component.myTemperatureControl', {
	extend : 'Ext.panel.Panel',
	myTemperatureControl : null,
	myValueControl : null,
	myTargetMode : null,
	myTemperatureLabel : null,
	myValueLabel : null,
	getHeight : function() {
		return 26;
	},
	constructor : function(config) {
		config.width = 700;
		config.border = false;
		this.myTargetMode = config.targetMode;

		var oBeginLabel = Ext.create('Ext.panel.Panel', {
			border : false,
			html : '<span style="color:#005138">' + config.label + ": </span>",
			width : 180
		});

		this.myTemperatureControl = Ext.create('Ext.form.field.Number', {
			labelWidth : 140,
			minValue : 3,
			maxValue : 35,
			step : 0.1,
			decimalSeparator : ',',
			decimalPrecision : 1,
            allowBlank : false,
			width : 60
		});

		this.myTemperatureLabel = Ext.create('Ext.panel.Panel', {
			padding : '2 0 0 5',
			border : false,
			html : '<span style="color:#005138">&deg;C</span>',
			width : 40
		});

		this.myValueControl = Ext.create('Ext.form.field.Number', {
			minValue : 0,
			maxValue : 100,
			step : 5,
			decimalPrecision : 0,
			width : 60
		});

		this.myValueLabel = Ext.create('Ext.panel.Panel', {
			padding : '2 0 0 5',
			border : false,
			html : '<span style="color:#005138">%</span>',
			width : 40
		});

        this.modeDescription = Ext.create('Ext.form.field.Display', {
            border : 0,
            height:18,
            flex:1,
            // width : 800,
            value : config.description
        });
        
		Ext.apply(this, {
			padding : '2 0 0 0',
			layout : 'hbox',
			height : 24,
			items : [ oBeginLabel, this.myTemperatureControl, this.myTemperatureLabel, this.myValueControl, this.myValueLabel, this.modeDescription ]
		});
		this.callParent(arguments);
	},
	reset : function() {
		var me = this;

		if (this.dssEditor.editControls['controlMode'].getValue() == 1) {
			this.myTemperatureLabel.setVisible(true);
			this.myTemperatureControl.setVisible(true);
			this.myTemperatureControl.setValue(null);
			this.myValueLabel.setVisible(false);
			this.myValueControl.setVisible(false);
			this.myValueControl.setValue(null);
		} else {
			this.myTemperatureLabel.setVisible(false);
			this.myTemperatureControl.setVisible(false);
			this.myTemperatureControl.setValue(null);
			this.myValueLabel.setVisible(true);
			this.myValueControl.setVisible(true);
			this.myValueControl.setValue(null);
		}
	},
	setValue : function(value) {
		var me = this;

		if (this.dssEditor.editControls['controlMode'].getValue() == 1) {
			this.myTemperatureLabel.setVisible(true);
			this.myTemperatureControl.setVisible(true);
			this.myTemperatureControl.setValue(value);
			this.myValueLabel.setVisible(false);
			this.myValueControl.setVisible(false);
			this.myValueControl.setValue(null);
		} else {
			this.myTemperatureLabel.setVisible(false);
			this.myTemperatureControl.setVisible(false);
			this.myTemperatureControl.setValue(null);
			this.myValueLabel.setVisible(true);
			this.myValueControl.setVisible(true);
			this.myValueControl.setValue(value);
		}

	},
	isValid : function() {
		var me = this;
		if (this.dssEditor.editControls['controlMode'].getValue() == 1)
			return this.myTemperatureControl.isValid();
		else
			return this.myValueControl.isValid();
	},
	getValue : function() {
		var me = this;

		if (this.dssEditor.editControls['controlMode'].getValue() == 1) {
			return this.myTemperatureControl.getValue();
		} else {
			return this.myValueControl.getValue();
		}
	}
});

Ext.define('DSS.addon.HeatingControl',
		{

			extend : 'DSS.addon.Framework',
			usableZones : [],

			possibleZones : [],
			tabControl : null,

			config : {
				appName : 'Heating Control',
				appId : 'hleedit',
				appVersion : '1.0.11',
				appIcon : 'default_icon.png'
			},
			getHelp : function() {
				return Ext.create('Ext.Component', {
                    whiteSpace : 'normal',
                    padding : '0 20 0 0', // top right bottom left
					loader : {
						url : 'locale/' + dss.staticDataModel.activeLanguage + '/help.html',
						autoLoad : true,
						disableCaching : false
					}
				});
			},
			getTimeString : function(v, fSeconds) {
				if (fSeconds == null)
					fSeconds = false;
				var sString = "";
				switch (Math.floor(v / (86400)) % 7) {
				case 0:
					sString += _("monday");
					break;
				case 1:
					sString += _("tuesday");
					break;
				case 2:
					sString += _("wednesday");
					break;
				case 3:
					sString += _("thursday");
					break;
				case 4:
					sString += _("friday");
					break;
				case 5:
					sString += _("saturday");
					break;
				case 6:
					sString += _("sunday");
					break;
				}
				sString += ", " + Math.floor(v / (3600)) % 24;
				sString += ":" + pad(Math.floor(v / (60)) % 60);
				if (fSeconds)
					sString += ":" + pad(Math.floor(v) % 60);
				return sString;
			},
			loadError : {   errorPresent : false,
                            errorDescr : ''
                        },
            startPanel : null,
            startPanelStatusText : null,
            startPanelStatusBar : null,
            tempPanelIntro : null,
			temperaturePanel : null,
			timePanel : null,
			generalPanel : null,
			sensorPanel : null,
			generalSettingsControl : {},
			timeApartmentEditor : null,
			timeNightEditor : null,
			timeNightValues : {},
			timeStore : null,
			timeValues : null,
			generalSettingsMissing : true,
			controllerDefined : false,
			zoneControlValues : null,
			overviewPanel:null,
            autoSelectMessage : [],
			heatingStore_controllerMode : null,
			heatingStore_referenceZone : null,
            setErrorState : function(state, descr) {
                if ((state) && (state == true)) {
                    this.loadError.errorPresent = state;
                    this.loadError.errorDescr = descr;
                } else {
                    this.loadError.errorPresent = false;
                    this.loadError.errorDescr = '';
                }
            },
			setNightModeVisibility : function(bCentralNightMode) {
				/*
				 * for (var iIndex=0;iIndex<this.temperaturePanel.columns.length;iIndex++) {
				 * if
				 * (this.temperaturePanel.columns[iIndex].dataIndex=="nominalTemperatureNight")
				 * this.temperaturePanel.columns[iIndex].setVisible(!bCentralNightMode); }
				 * this.newNightButton.setDisabled(bCentralNightMode);
				 */
			},
			updateTimeStore : function() {
				var me = this;
				var nightValue = [];
				var startMonday = 0;

				if (this.timeNightValues['SU'] != null) {
					if (this.timeNightValues['SU'].to<(12*60*60)) {
						startMonday = this.timeNightValues['SU'].to;						
						if (this.timeNightValues['SU'].from<(12*60*60)) {
							nightValue.push({
								duration : this.timeNightValues['SU'].from,
								type : OperationModes.nothing
							});
							startMonday-=this.timeNightValues['SU'].from;
						}												
						nightValue.push({
							duration : startMonday,
							type : OperationModes.Night
						});
						startMonday = this.timeNightValues['SU'].to;						
					}
				}

				if (this.timeNightValues['MO'] != null) {
					var diffTime = this.timeNightValues['MO'].from - startMonday;
					if (this.timeNightValues['MO'].from <(12*60*60))
							diffTime+=24*60*60;
					nightValue.push({
						duration : diffTime,
						type : OperationModes.nothing
					});
					
					if (this.timeNightValues['MO'].to>this.timeNightValues['MO'].from) {
						diffTime = (this.timeNightValues['MO'].to  - this.timeNightValues['MO'].from);
					} else {
						diffTime = ((this.timeNightValues['MO'].to) + ((24 * 60 * 60) - this.timeNightValues['MO'].from));	
					}
					
					
					nightValue.push({
						duration : diffTime,
						type : OperationModes.Night
					});
					startMonday = this.timeNightValues['MO'].to + (24 * 60 * 60);
					if (this.timeNightValues['MO'].to>(12*60*60))
						startMonday-=(24 * 60 * 60);
				}

				if (this.timeNightValues['TU'] != null) {
					var diffTime = (this.timeNightValues['TU'].from + (24 * 60 * 60)) - startMonday;
					if (this.timeNightValues['TU'].from <(12*60*60))
							diffTime+=24*60*60;
					nightValue.push({
						duration : diffTime,
						type : OperationModes.nothing
					});

					if (this.timeNightValues['TU'].to>this.timeNightValues['TU'].from) {
						diffTime = (this.timeNightValues['TU'].to  - this.timeNightValues['TU'].from);
					} else {
						diffTime = ((this.timeNightValues['TU'].to) + ((24 * 60 * 60) - this.timeNightValues['TU'].from));	
					}

					nightValue.push({
						duration : diffTime,
						type : OperationModes.Night
					});
					startMonday = this.timeNightValues['TU'].to + (2 * 24 * 60 * 60);
					if (this.timeNightValues['TU'].to>(12*60*60))
						startMonday-=(24 * 60 * 60);
				}

				if (this.timeNightValues['WE'] != null) {
					var diffTime = (this.timeNightValues['WE'].from + 2 * 24 * 60 * 60) - startMonday;
					if (this.timeNightValues['WE'].from <(12*60*60))
						diffTime+=24*60*60;
					nightValue.push({
						duration : diffTime,
						type : OperationModes.nothing
					});

					if (this.timeNightValues['WE'].to>this.timeNightValues['WE'].from) {
						diffTime = (this.timeNightValues['WE'].to  - this.timeNightValues['WE'].from);
					} else {
						diffTime = ((this.timeNightValues['WE'].to) + ((24 * 60 * 60) - this.timeNightValues['WE'].from));	
					}

					nightValue.push({
						duration : diffTime,
						type : OperationModes.Night
					});
					startMonday = this.timeNightValues['WE'].to + (3 * 24 * 60 * 60);
					if (this.timeNightValues['WE'].to>(12*60*60))
						startMonday-=(24 * 60 * 60);
				}

				if (this.timeNightValues['TH'] != null) {
					var diffTime = (this.timeNightValues['TH'].from + 3 * 24 * 60 * 60) - startMonday;
					if (this.timeNightValues['TH'].from <(12*60*60))
						diffTime+=24*60*60;

					nightValue.push({
						duration : diffTime,
						type : OperationModes.nothing
					});
					
					if (this.timeNightValues['TH'].to>this.timeNightValues['TH'].from) {
						diffTime = (this.timeNightValues['TH'].to  - this.timeNightValues['TH'].from);
					} else {
						diffTime = ((this.timeNightValues['TH'].to) + ((24 * 60 * 60) - this.timeNightValues['TH'].from));	
					}
					
					nightValue.push({
						duration : diffTime,
						type : OperationModes.Night
					});
					startMonday = this.timeNightValues['TH'].to + (4 * 24 * 60 * 60);
					if (this.timeNightValues['TH'].to>(12*60*60))
						startMonday-=(24 * 60 * 60);
				}

				if (this.timeNightValues['FR'] != null) {
					var diffTime = (this.timeNightValues['FR'].from + 4 * 24 * 60 * 60) - startMonday;
					if (this.timeNightValues['FR'].from <(12*60*60))
						diffTime+=24*60*60;

					nightValue.push({
						duration : diffTime,
						type : OperationModes.nothing
					});

					if (this.timeNightValues['FR'].to>this.timeNightValues['FR'].from) {
						diffTime = (this.timeNightValues['FR'].to  - this.timeNightValues['FR'].from);
					} else {
						diffTime = ((this.timeNightValues['FR'].to) + ((24 * 60 * 60) - this.timeNightValues['FR'].from));	
					}
					
					nightValue.push({
						duration : diffTime,
						type : OperationModes.Night
					});
					startMonday = this.timeNightValues['FR'].to + (5 * 24 * 60 * 60);
					if (this.timeNightValues['FR'].to>(12*60*60))
						startMonday-=(24 * 60 * 60);
				}

				if (this.timeNightValues['SA'] != null) {
					var diffTime = (this.timeNightValues['SA'].from + 5 * 24 * 60 * 60) - startMonday;
					if (this.timeNightValues['SA'].from <(12*60*60))
						diffTime+=24*60*60;

					nightValue.push({
						duration : diffTime,
						type : OperationModes.nothing
					});

					if (this.timeNightValues['SA'].to>this.timeNightValues['SA'].from) {
						diffTime = (this.timeNightValues['SA'].to  - this.timeNightValues['SA'].from);
					} else {
						diffTime = ((this.timeNightValues['SA'].to) + ((24 * 60 * 60) - this.timeNightValues['SA'].from));	
					}

					nightValue.push({
						duration : diffTime,
						type : OperationModes.Night
					});
					startMonday = this.timeNightValues['SA'].to + (6 * 24 * 60 * 60);
					if (this.timeNightValues['SA'].to>(12*60*60))
						startMonday-=(24 * 60 * 60);
				}

				if (this.timeNightValues['SU'] != null) {
					if (this.timeNightValues['SU'].from>(12*60*60)) {
						var diffTime = (this.timeNightValues['SU'].from + 6 * 24 * 60 * 60) - startMonday;
	
						nightValue.push({
							duration : diffTime,
							type : OperationModes.nothing
						});
						
						startMonday+=diffTime;
						
						if (this.timeNightValues['SU'].to>this.timeNightValues['SU'].from) {
							diffTime = (this.timeNightValues['SU'].to  - this.timeNightValues['SU'].from);
						} else {
							diffTime = ((this.timeNightValues['SU'].to) + ((24 * 60 * 60) - this.timeNightValues['SU'].from));	
						}
	
						if (startMonday+diffTime>(7 * 24 * 60 * 60))
							diffTime=(7 * 24 * 60 * 60)-startMonday;
						
						nightValue.push({
							duration : diffTime,
							type : OperationModes.Night
						});
						
						if (this.timeNightValues['SU'].to>(12*60*60)) {
							nightValue.push({
								duration : ((7 * 24 * 60 * 60) - startMonday-diffTime),
								type : OperationModes.nothing
							});
						}
                    } else {
                        nightValue.push({
                            duration : ((7 * 24 * 60 * 60) - startMonday),
                            type : OperationModes.nothing
                        });
                    }
                    startMonday = (7 * 24 * 60 * 60);
                }
                nightValue.push({
						duration : ((7 * 24 * 60 * 60) - startMonday),
						type : OperationModes.nothing
					});
		

				this.timeValues.push({
					name : _('Night Mode'),
					id : -1,
					val : nightValue
				});
			
				/*
				 * var fApartmentExist = false; for ( var iIndex = 0; iIndex <
				 * this.timeValues.length; iIndex++) { if
				 * (this.timeValues[iIndex].id == 0) fApartmentExist = true; }
				 * if (!fApartmentExist) this.timeValues.push({ name :
				 * 'Apartment', id : 0, val : [ { duration : (7 * 24 * 60 * 60),
				 * type : OperationModes.Eco } ] });
				 */
				var maxValues = 0;
				for ( var iIndex = 0; iIndex < this.timeValues.length; iIndex++) {
					var timeSeries = this.timeValues[iIndex];
					if (timeSeries.val.length > maxValues)
						maxValues = timeSeries.val.length;
				}
				var fields = [ 'label', 'zoneID' ];
				var fields2 = [];
				for ( var iIndex = 0; iIndex < maxValues; iIndex++) {
					fields.push('val' + iIndex);
					fields.push('typ' + iIndex);
					fields.push('tID' + iIndex);
					fields2.push('val' + iIndex);
				}

				var data = [];
				for ( var iIndex = 0; iIndex < this.timeValues.length; iIndex++) {
					var dateObject = {
						'label' : this.timeValues[iIndex].name,
						'zoneID' : this.timeValues[iIndex].id
					};
					var timeSeries = this.timeValues[iIndex];
					var kIndex = 0;
					for (; kIndex < timeSeries.val.length; kIndex++) {
						dateObject['val' + kIndex] = timeSeries.val[kIndex].duration;
						dateObject['typ' + kIndex] = timeSeries.val[kIndex].type;
						dateObject['tID' + kIndex] = timeSeries.val[kIndex].id;
					}
					for (; kIndex < maxValues; kIndex++) {
						dateObject['val' + kIndex] = 0;
						dateObject['typ' + kIndex] = -1;
					}
					data.push(dateObject);
				}
				if (me.timeStore == null) {
					me.timeStore = Ext.create('Ext.data.Store', {
						fields : fields,
						data : data,
						sorters : [ {
							sorterFn : function(o1, o2) {
								var iSort1 = o1.get('zoneID');
								var iSort2 = o2.get('zoneID');
								if (iSort1 == 0) {
									return 1;
								}
								if (iSort2 == 0) {
									return -1;
								}
								if (iSort1 == -1) {
									if (iSort2 == 0)
										return -1;
									else
										return 1;
								}
								if (iSort2 == -1) {
									if (iSort1 == 0)
										return 1;
									else
										return -1;
								}
								return dss.staticDataModel.getNameForZone(iSort1) < dss.staticDataModel.getNameForZone(iSort2);
							}
						} ]
					});

					var chart = Ext.create('Ext.chart.Chart', {
						// animate : true,
						shadow : false,
						store : me.timeStore,
						axes : [ {
							type : 'Numeric',
							position : 'bottom',
							fields : fields2,
							title : false,
							grid : true,
							majorTickSteps : 7,
							minimum : 0,
							maximum : (7 * 24 * 60 * 60),
							label : {
								renderer : me.getTimeString
							}
						}, {
							type : 'Category',
							position : 'left',
							fields : [ 'label' ],
							title : false
						} ],
						series : [ {
							type : 'bar',
							axis : 'bottom',
							gutter : 80,
							xField : 'room',
							yField : fields2,
							stacked : true,
							renderer : function(sprite, record, attributes, index, store) {
								return attributes;
							},
							tips : {
								trackMouse : true,
								width : 300,
								height : 35,
								layout : {
									type : 'hbox',
									align : 'left'
								},
								renderer : function(storeItem, item) {
									var baseIndex = parseInt(item.yField.substr(3), 10);
									var iStarttime = 0;
									for ( var iIndex = 0; iIndex < baseIndex; iIndex++) {
										iStarttime += storeItem.data['val' + iIndex];
									}
									var iStoptime = iStarttime + item.value[1];
									var typ = storeItem.get('typ' + baseIndex);
									var sModeString = "";
									switch (typ) {
									case OperationModes.transparent:
									case OperationModes.nothing:
										sModeString = _('(inactive - apartmentwide mode is active)');
										break;
									case OperationModes.Comfort:
										sModeString = _('Comfort Mode');
										break;
									case OperationModes.Eco:
										sModeString = _('Eco Mode');
										break;
									case OperationModes.NotUsed:
										sModeString = _('NotUsed Mode');
										break;
									case OperationModes.Night:
										sModeString = _('Night Mode');
										break;
									}
									this.setTitle(_('Mode:') + ' ' + sModeString + '<br>' + me.getTimeString(iStarttime) + " - " + me.getTimeString(iStoptime));
								}
							},
							listeners : {
								itemmousedown : function(evtObject) {
									var oRecord = evtObject.storeItem;
									var oItem = evtObject.yField;
									var itemIndex = oItem.substr(3);
									var iZoneID = oRecord.get('zoneID');
									if (iZoneID == -1) {
										me.timeNightEditor.show();
										me.timeNightEditor.setValue(me.timeNightValues);
									}
									if (iZoneID == 0) {
										var iType = oRecord.get('typ' + itemIndex);
										if (iType == OperationModes.Eco) {
											// clicked on eco -> new Comfort
											// zone
											me.timeApartmentEditor.setValue({
												from : null,
												fromID : null,
												to : null,
												toID : null
											});
											me.timeApartmentEditor.show();
										} else {
											var iStart = 0;
											var iEnd = 0;
											for ( var iIndex = 0; iIndex < parseInt(itemIndex, 10); iIndex++) {
												iStart += oRecord.get('val' + iIndex);
											}
											iEnd = iStart + oRecord.get(oItem);
											me.timeApartmentEditor.setValue({
												from : iStart,
												fromID : oRecord.get('tID' + parseInt(itemIndex, 10)),
												to : iEnd,
												toID : oRecord.get('tID' + (parseInt(itemIndex, 10) + 1))
											});
											me.timeApartmentEditor.show();
										}
									}
									if (iZoneID > 0) {
										/*
										 * var iType = oRecord.get('typ' +
										 * itemIndex); if (iType ==
										 * OperationModes.NotUsed) { var iStart =
										 * 0; var iEnd = 0; for ( var iIndex =
										 * 0; iIndex < parseInt(itemIndex, 10);
										 * iIndex++) { iStart +=
										 * oRecord.get('val' + iIndex); } iEnd =
										 * iStart + oRecord.get(oItem);
										 * me.timeZoneEditor.setValue({ from :
										 * iStart, fromID : oRecord.get('tID' +
										 * parseInt(itemIndex, 10)), to : iEnd,
										 * toID : oRecord.get('tID' +
										 * (parseInt(itemIndex, 10) + 1)),
										 * zoneID : iZoneID }); } else {
										 * me.timeZoneEditor.setValue({ from :
										 * null, fromID : null, to : null, toID :
										 * null, zoneID : iZoneID }); }
										 */
										me.timeZoneEditor.setValue(iZoneID);
										me.timeZoneEditor.show();

									}
								}
							}
						} ]
					});

					return chart;
				} else {
					var chart = me.timePanel.items.items[0];
					me.timeStore.removeAll();
					me.timeStore.model.setFields(fields);
					var series = chart.series.first();
					series.yField = fields2;
					me.timeStore.add(data);
				}

			},
			makeTimePanel : function() {
				var me = this;
				me.timeNightEditor = Ext.create('DSS.addon.TimeNightEditor',
												{ base: me });
				// me.timeNightEditor.base = me;
				me.timeApartmentEditor = Ext.create('DSS.addon.TimeApartmentEditor', {});
				me.timeApartmentEditor.base = me;
				me.timeZoneEditor = Ext.create('DSS.addon.SingularZoneEditor',
												{ base: me });

				me.loadTimedSettings();

				/*
				 * var newApartmentButton = Ext.create('Ext.button.Button', {
				 * text : _('Define new Comfort period'), iconCls :
				 * 'icon-add-action', handler : function() {
				 * me.timeApartmentEditor.setValue({ from : null, fromID : null,
				 * to : null, toID : null }); me.timeApartmentEditor.show(); } })
				 */;
				me.newNightButton = Ext.create('Ext.button.Button', {
					text : _('Define Night period'),
					iconCls : 'icon-add-action',
					handler : function() {
						me.timeNightEditor.show();
						me.timeNightEditor.setValue(me.timeNightValues);
					}
				});
				var newRoomButton = new Ext.create('Ext.button.Button', {
					text : _('Define new unused period'),
					iconCls : 'icon-add-action',
					handler : function() {
						me.timeZoneEditor.setValue(null);
						me.timeZoneEditor.show();
					}
				});

				var refreshButton= new Ext.create('Ext.button.Button',{
					text:_('Refresh'),
    				handler: function(){
    					me.loadTimedSettings();
    					me.updateTimeStore();
    				},
    				icon: dss.staticDataModel.baseImagePath + 'buttonIcons/refresh.png'
				});
				
				this.timePanel = Ext.create('widget.panel', {
					layout : 'fit',
					tbar : [ me.newNightButton, ' ', newRoomButton ],
					bbar : ['->' , refreshButton]
				});

				this.timePanel.add(me.updateTimeStore());

			},
			removeApartmentTime : function(value) {
				dss.raiseEvent('heating-controller.time-profile', {
					actions : 'delete',
					id : Ext.JSON.encode([ value.fromID, value.toID ])
				});
				this.waitAndReloadTimedSettings();
			},
			saveZoneTime : function(value) {
				if (value.zoneID == null)
					return;
				var fromWD = "";
				var toWD = "";

				switch (Math.floor(value.from / (24 * 60 * 60))) {
				case 0:
					fromWD = "MO";
					break;
				case 1:
					fromWD = "TU";
					break;
				case 2:
					fromWD = "WE";
					break;
				case 3:
					fromWD = "TH";
					break;
				case 4:
					fromWD = "FR";
					break;
				case 5:
					fromWD = "SA";
					break;
				case 6:
					fromWD = "SU";
					break;
				}
				;
				switch (Math.floor(value.to / (24 * 60 * 60))) {
				case 0:
					toWD = "MO";
					break;
				case 1:
					toWD = "TU";
					break;
				case 2:
					toWD = "WE";
					break;
				case 3:
					toWD = "TH";
					break;
				case 4:
					toWD = "FR";
					break;
				case 5:
					toWD = "SA";
					break;
				case 6:
					toWD = "SU";
					break;
				}
				;

				var oParameter = [ {
					id : value.fromID,
					desiredMode : OperationModes.NotUsed,
					target : value.zoneID,
					time : {
						offset : (value.from % (24 * 60 * 60)),
						timeBase : 'daily'
					},
					recurrence : {
						recurrenceBase : 'weekly',
						recurrenceOffset : fromWD
					}
				}, {
					id : value.toID,
					resetingMode : OperationModes.NotUsed,
					target : value.zoneID,
					time : {
						offset : (value.to % (24 * 60 * 60)),
						timeBase : 'daily'
					},
					recurrence : {
						recurrenceBase : 'weekly',
						recurrenceOffset : toWD
					}
				} ];
				if (oParameter.length > 0) {
					dss.raiseEvent('heating-controller.time-profile', {
						actions : 'save',
						value : Ext.JSON.encode(oParameter)
					});
					this.waitAndReloadTimedSettings();
				}
			},
			removeTimeline : function(zoneID) {
				dss.raiseEvent('heating-controller.time-profile', {
					actions : 'deleteTimeline',
					zoneID : zoneID
				});
				this.waitAndReloadTimedSettings();
			},
			removeZoneTime : function(value) {
				dss.raiseEvent('heating-controller.time-profile', {
					actions : 'delete',
					id : Ext.JSON.encode([ value.fromID, value.toID ])
				});
				this.waitAndReloadTimedSettings();
			},
			saveApartmentTime : function(value) {
				var fromWD = "";
				var toWD = "";
				switch (Math.floor(value.from / (24 * 60 * 60))) {
				case 0:
					fromWD = "MO";
					break;
				case 1:
					fromWD = "TU";
					break;
				case 2:
					fromWD = "WE";
					break;
				case 3:
					fromWD = "TH";
					break;
				case 4:
					fromWD = "FR";
					break;
				case 5:
					fromWD = "SA";
					break;
				case 6:
					fromWD = "SU";
					break;
				}
				;
				switch (Math.floor(value.to / (24 * 60 * 60))) {
				case 0:
					toWD = "MO";
					break;
				case 1:
					toWD = "TU";
					break;
				case 2:
					toWD = "WE";
					break;
				case 3:
					toWD = "TH";
					break;
				case 4:
					toWD = "FR";
					break;
				case 5:
					toWD = "SA";
					break;
				case 6:
					toWD = "SU";
					break;
				}
				;

				var oParameter = [ {
					id : value.fromID,
					desiredMode : OperationModes.Comfort,
					target : 0,
					time : {
						offset : (value.from % (24 * 60 * 60)),
						timeBase : 'daily'
					},
					recurrence : {
						recurrenceBase : 'weekly',
						recurrenceOffset : fromWD
					}
				}, {
					id : value.toID,
					desiredMode : OperationModes.Eco,
					target : 0,
					time : {
						offset : (value.to % (24 * 60 * 60)),
						timeBase : 'daily'
					},
					recurrence : {
						recurrenceBase : 'weekly',
						recurrenceOffset : toWD
					}
				} ];
				if (oParameter.length > 0) {
					dss.raiseEvent('heating-controller.time-profile', {
						actions : 'save',
						value : Ext.JSON.encode(oParameter)
					});
					this.waitAndReloadTimedSettings();
				}
			},
			saveNightTime : function(value) {
				var oParameter = [];
				
				
				if (value.MO != null) {
					var dayFrom='MO';
					if (value.MO.from<12*60*60)
						dayFrom='TU';
					var dayTo='TU';
					if (value.MO.to>12*60*60)
						dayTo='MO';

					oParameter.push({
						id : value.MO.fromID,
						desiredMode : OperationModes.Night,
						target : 0,
						time : {
							offset : value.MO.from,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : dayFrom
						}
					});
					oParameter.push({
						id : value.MO.toID,
						resetingMode : OperationModes.Night,
						target : 0,
						time : {
							offset : value.MO.to,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : dayTo
						}
					});
				} else {
					oParameter.push({
						id : null,
						desiredMode : OperationModes.Night,
						target : 0,
						time : {
							offset : (12*60*60),
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : 'MO'
						},
						disabled:true
					});
					oParameter.push({
						id : null,
						resetingMode : OperationModes.Night,
						target : 0,
						time : {
							offset : (12*60*60)-1,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : 'TU'
						},
						disabled:true
					});
				}
				if (value.TU != null) {
					var dayFrom='TU';
					if (value.TU.from<12*60*60)
						dayFrom='WE';
					var dayTo='WE';
					if (value.TU.to>12*60*60)
						dayTo='TU';
					
					oParameter.push({
						id : value.TU.fromID,
						desiredMode : OperationModes.Night,
						target : 0,
						time : {
							offset : value.TU.from,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : dayFrom
						}
					});
					oParameter.push({
						id : value.TU.toID,
						resetingMode : OperationModes.Night,
						target : 0,
						time : {
							offset : value.TU.to,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : dayTo
						}
					});
				} else {
					oParameter.push({
						id : null,
						desiredMode : OperationModes.Night,
						target : 0,
						time : {
							offset : (12*60*60),
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : 'TU'
						},
						disabled:true
					});
					oParameter.push({
						id : null,
						resetingMode : OperationModes.Night,
						target : 0,
						time : {
							offset : (12*60*60)-1,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : 'WE'
						},
						disabled:true
					});
				}
				
				if (value.WE != null) {
					var dayFrom='WE';
					if (value.WE.from<12*60*60)
						dayFrom='TH';
					var dayTo='TH';
					if (value.WE.to>12*60*60)
						dayTo='WE';
					oParameter.push({
						id : value.WE.fromID,
						desiredMode : OperationModes.Night,
						target : 0,
						time : {
							offset : value.WE.from,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : dayFrom
						}
					});
					oParameter.push({
						id : value.WE.toID,
						resetingMode : OperationModes.Night,
						target : 0,
						time : {
							offset : value.WE.to,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : dayTo
						}
					});
				} else {
					oParameter.push({
						id : null,
						desiredMode : OperationModes.Night,
						target : 0,
						time : {
							offset : (12*60*60),
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : 'WE'
						},
						disabled:true
					});
					oParameter.push({
						id : null,
						resetingMode : OperationModes.Night,
						target : 0,
						time : {
							offset : (12*60*60)-1,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : 'TH'
						},
						disabled:true
					});
				}
				if (value.TH != null) {
					var dayFrom='TH';
					if (value.TH.from<12*60*60)
						dayFrom='FR';
					var dayTo='FR';
					if (value.TH.to>12*60*60)
						dayTo='TH';
					oParameter.push({
						id : value.TH.fromID,
						desiredMode : OperationModes.Night,
						target : 0,
						time : {
							offset : value.TH.from,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : dayFrom
						}
					});
					oParameter.push({
						id : value.TH.toID,
						resetingMode : OperationModes.Night,
						target : 0,
						time : {
							offset : value.TH.to,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : dayTo
						}
					});
				} else {
					oParameter.push({
						id : null,
						desiredMode : OperationModes.Night,
						target : 0,
						time : {
							offset : (12*60*60),
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : 'TH'
						},
						disabled:true
					});
					oParameter.push({
						id : null,
						resetingMode : OperationModes.Night,
						target : 0,
						time : {
							offset : (12*60*60)-1,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : 'FR'
						},
						disabled:true
					});
				}
				if (value.FR != null) {
					var dayFrom='FR';
					if (value.FR.from<12*60*60)
						dayFrom='SA';
					var dayTo='SA';
					if (value.FR.to>12*60*60)
						dayTo='FR';
					oParameter.push({
						id : value.FR.fromID,
						desiredMode : OperationModes.Night,
						target : 0,
						time : {
							offset : value.FR.from,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : dayFrom
						}
					});
					oParameter.push({
						id : value.FR.toID,
						resetingMode : OperationModes.Night,
						target : 0,
						time : {
							offset : value.FR.to,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : dayTo
						}
					});
				} else {
					oParameter.push({
						id : null,
						desiredMode : OperationModes.Night,
						target : 0,
						time : {
							offset : (12*60*60),
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : 'FR'
						},
						disabled:true
					});
					oParameter.push({
						id : null,
						resetingMode : OperationModes.Night,
						target : 0,
						time : {
							offset : (12*60*60)-1,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : 'SA'
						},
						disabled:true
					});
				}
				if (value.SA != null) {
					var dayFrom='SA';
					if (value.SA.from<12*60*60)
						dayFrom='SU';
					var dayTo='SU';
					if (value.SA.to>12*60*60)
						dayTo='SA';
					oParameter.push({
						id : value.SA.fromID,
						desiredMode : OperationModes.Night,
						target : 0,
						time : {
							offset : value.SA.from,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : dayFrom
						}
					});
					oParameter.push({
						id : value.SA.toID,
						resetingMode : OperationModes.Night,
						target : 0,
						time : {
							offset : value.SA.to,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : dayTo
						}
					});
				} else {
					oParameter.push({
						id : null,
						desiredMode : OperationModes.Night,
						target : 0,
						time : {
							offset : (12*60*60),
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : 'SA'
						},
						disabled:true
					});
					oParameter.push({
						id : null,
						resetingMode : OperationModes.Night,
						target : 0,
						time : {
							offset : (12*60*60)-1,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : 'SU'
						},
						disabled:true
					});
				}
				if (value.SU != null) {
					var dayFrom='SU';
					if (value.SU.from<12*60*60)
						dayFrom='MO';
					var dayTo='MO';
					if (value.SU.to>12*60*60)
						dayTo='SU';
					oParameter.push({
						id : value.SU.fromID,
						desiredMode : OperationModes.Night,
						target : 0,
						time : {
							offset : value.SU.from,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : dayFrom
						}
					});
					oParameter.push({
						id : value.SU.toID,
						resetingMode : OperationModes.Night,
						target : 0,
						time : {
							offset : value.SU.to,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : dayTo
						}
					});
				} else {
					oParameter.push({
						id : null,
						desiredMode : OperationModes.Night,
						target : 0,
						time : {
							offset : (12*60*60),
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : 'SU'
						},
						disabled:true
					});
					oParameter.push({
						id : null,
						resetingMode : OperationModes.Night,
						target : 0,
						time : {
							offset : (12*60*60)-1,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : 'MO'
						},
						disabled:true
					});
				}
				if (oParameter.length > 0) {
					dss.raiseEvent('heating-controller.time-profile', {
						actions : 'save',
						value : Ext.JSON.encode(oParameter)
					});
					this.waitAndReloadTimedSettings();
				}
			},
			saveSingularyTime : function(value) {
				var oParameter = [];
				var zoneID=value.zoneID;
				if (value.MO != null) {
					oParameter.push({
						id : value.MO.fromID,
						desiredMode : OperationModes.NotUsed,
						target : zoneID,
						time : {
							offset : value.MO.from,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : 'MO'
						}
					});
					oParameter.push({
						id : value.MO.toID,
						resetingMode : OperationModes.NotUsed,
						target : zoneID,
						time : {
							offset : value.MO.to,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : 'MO'
						}
					});
				} else {
					oParameter.push({
						id : null,
						desiredMode : OperationModes.NotUsed,
						target : zoneID,
						time : {
							offset : 0,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : 'MO'
						},
						disabled:true
					});
					oParameter.push({
						id : null,
						resetingMode : OperationModes.NotUsed,
						target : zoneID,
						time : {
							offset : (24*60*60)-1,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : 'MO'
						},
						disabled:true
					});
				}
				if (value.TU != null) {
					oParameter.push({
						id : value.TU.fromID,
						desiredMode : OperationModes.NotUsed,
						target : zoneID,
						time : {
							offset : value.TU.from,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : 'TU'
						}
					});
					oParameter.push({
						id : value.TU.toID,
						resetingMode : OperationModes.NotUsed,
						target : zoneID,
						time : {
							offset : value.TU.to,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : 'TU'
						}
					});
				} else {
					oParameter.push({
						id : null,
						desiredMode : OperationModes.NotUsed,
						target : zoneID,
						time : {
							offset : 0,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : 'TU'
						},
						disabled:true
					});
					oParameter.push({
						id : null,
						resetingMode : OperationModes.NotUsed,
						target : zoneID,
						time : {
							offset : (24*60*60)-1,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : 'TU'
						},
						disabled:true
					});
				}

				if (value.WE != null) {
					oParameter.push({
						id : value.WE.fromID,
						desiredMode : OperationModes.NotUsed,
						target : zoneID,
						time : {
							offset : value.WE.from,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : 'WE'
						}
					});
					oParameter.push({
						id : value.WE.toID,
						resetingMode : OperationModes.NotUsed,
						target : zoneID,
						time : {
							offset : value.WE.to,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : 'WE'
						}
					});
				} else {
					oParameter.push({
						id : null,
						desiredMode : OperationModes.NotUsed,
						target : zoneID,
						time : {
							offset : 0,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : 'WE'
						},
						disabled:true
					});
					oParameter.push({
						id : null,
						resetingMode : OperationModes.NotUsed,
						target : zoneID,
						time : {
							offset : (24*60*60)-1,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : 'WE'
						},
						disabled:true
					});
				}

				if (value.TH != null) {
					oParameter.push({
						id : value.TH.fromID,
						desiredMode : OperationModes.NotUsed,
						target : zoneID,
						time : {
							offset : value.TH.from,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : 'TH'
						}
					});
					oParameter.push({
						id : value.TH.toID,
						resetingMode : OperationModes.NotUsed,
						target : zoneID,
						time : {
							offset : value.TH.to,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : 'TH'
						}
					});
				} else {
					oParameter.push({
						id : null,
						desiredMode : OperationModes.NotUsed,
						target : zoneID,
						time : {
							offset : 0,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : 'TH'
						},
						disabled:true
					});
					oParameter.push({
						id : null,
						resetingMode : OperationModes.NotUsed,
						target : zoneID,
						time : {
							offset : (24*60*60)-1,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : 'TH'
						},
						disabled:true
					});
				}

				if (value.FR != null) {
					oParameter.push({
						id : value.FR.fromID,
						desiredMode : OperationModes.NotUsed,
						target : zoneID,
						time : {
							offset : value.FR.from,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : 'FR'
						}
					});
					oParameter.push({
						id : value.FR.toID,
						resetingMode : OperationModes.NotUsed,
						target : zoneID,
						time : {
							offset : value.FR.to,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : 'FR'
						}
					});
				} else {
					oParameter.push({
						id : null,
						desiredMode : OperationModes.NotUsed,
						target : zoneID,
						time : {
							offset : 0,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : 'FR'
						},
						disabled:true
					});
					oParameter.push({
						id : null,
						resetingMode : OperationModes.NotUsed,
						target : zoneID,
						time : {
							offset : (24*60*60)-1,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : 'FR'
						},
						disabled:true
					});
				}

				if (value.SA != null) {
					oParameter.push({
						id : value.SA.fromID,
						desiredMode : OperationModes.NotUsed,
						target : zoneID,
						time : {
							offset : value.SA.from,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : 'SA'
						}
					});
					oParameter.push({
						id : value.SA.toID,
						resetingMode : OperationModes.NotUsed,
						target : zoneID,
						time : {
							offset : value.SA.to,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : 'SA'
						}
					});
				} else {
					oParameter.push({
						id : null,
						desiredMode : OperationModes.NotUsed,
						target : zoneID,
						time : {
							offset : 0,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : 'SA'
						},
						disabled:true
					});
					oParameter.push({
						id : null,
						resetingMode : OperationModes.NotUsed,
						target : zoneID,
						time : {
							offset : (24*60*60)-1,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : 'SA'
						},
						disabled:true
					});
				}

				if (value.SU != null) {
					oParameter.push({
						id : value.SU.fromID,
						desiredMode : OperationModes.NotUsed,
						target : zoneID,
						time : {
							offset : value.SU.from,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : 'SU'
						}
					});
					oParameter.push({
						id : value.SU.toID,
						resetingMode : OperationModes.NotUsed,
						target : zoneID,
						time : {
							offset : value.SU.to,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : 'SU'
						}
					});
				} else {
					oParameter.push({
						id : null,
						desiredMode : OperationModes.NotUsed,
						target : zoneID,
						time : {
							offset : 0,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : 'SU'
						},
						disabled:true
					});
					oParameter.push({
						id : null,
						resetingMode : OperationModes.NotUsed,
						target : zoneID,
						time : {
							offset : (24*60*60)-1,
							timeBase : 'daily'
						},
						recurrence : {
							recurrenceBase : 'weekly',
							recurrenceOffset : 'SU'
						},
						disabled:true
					});
				}

				if (oParameter.length > 0) {
					dss.raiseEvent('heating-controller.time-profile', {
						actions : 'deleteTimeline',
						zoneID : zoneID,
						noAnswer:true
					});
					dss.raiseEvent('heating-controller.time-profile', {
						actions : 'save',
						value : Ext.JSON.encode(oParameter)
					});
					this.waitAndReloadTimedSettings('save');
				}
			},			
			formatTemperature : function(val, controlMode) {
				var text;
				switch (controlMode) {
				case ControlModes.cControlModeOff:
					text = '-';
					break;
				case ControlModes.cControlModePID:
					if (val === null)
						return '-';
					if (("" + val) == "")
						return '-';
					val = Math.floor((val + 0.05) * 10) / 10;
					text = ("" + val).replace('.', ',') + " &deg;C";
					break;
				case ControlModes.cControlModeZoneFollower:
					text = '-';
					break;
				case ControlModes.cControlModeFixedValue:
					if (val === null)
						return '-';
					if (("" + val) == "")
						return '-';
					text = ("" + val).replace('.', ',') + " %";
					break;
				default:
					if (val === null)
						return '-';
					if (("" + val) == "")
						return '-';
					val = Math.floor((val + 0.05) * 10) / 10;
					text = ("" + val).replace('.', ',') + " &deg;C";
					break;
					break;
				}
				return text;
			},
			makeTemperaturePanel : function() {
				function formatCell(text, valid) {
					if (!valid) {
						text = '<span style="color:#aaaaaa;">' + text + '</span>';
					}
					return text;
				}
				;
				function createTooltip(record) {
					var sText = '';
					switch (record.get('effectiveNominalTemperature')) {
					case OperationModes.Comfort:
						if (record.get('nominalTemperatureComfort') == record.get('effectiveNominalTemperature')) {
							sText = _('current operation mode is comfort');
						} else {
							sText = _('current nominal temperature is set temporary');
						}
						break;
					case OperationModes.Eco:
						if (record.get('nominalTemperatureEco') == record.get('effectiveNominalTemperature')) {
							sText = _('current operation mode is economy');
						} else {
							sText = _('current nominal temperature is set temporary');
						}
						break;
					case OperationModes.NotUsed:
						if (record.get('nominalTemperatureNotUsed') == record.get('effectiveNominalTemperature')) {
							sText = _('current operation mode is not used');
						} else {
							sText = _('current nominal temperature is set temporary');
						}
						break;
					case OperationModes.Night:
						if (record.get('nominalTemperatureNight') == record.get('effectiveNominalTemperature')) {
							sText = _('current operation mode is night');
						} else {
							sText = _('current nominal temperature is set temporary');
						}
						break;
					case OperationModes.Holiday:
						if (record.get('nominalTemperatureHoliday') == record.get('effectiveNominalTemperature')) {
							sText = _('current operation mode is holiday');
						} else {
							sText = _('current nominal temperature is set temporary');
						}
						break;
					}
					return '<table><tr><td>' + sText + '</td></tr></table>';
				}
				;
                var roomNameField = Ext.create('Ext.form.field.Display', {
                            border : 0,
                            height:18,
                            fieldStyle: {'fontWeight' : 'bold'}                
                });
				var comfortField = Ext.create('DSS.component.myTemperatureControl', {
					targetMode : 'nominalTemperatureComfort',
					label : _('Comfort Mode'),
                    description : _('used in normal operation when comfort mode is selected')
				});
				var ecoField = Ext.create('DSS.component.myTemperatureControl', {
					targetMode : 'nominalTemperatureEco',
					label : _('Eco Mode'),
                    description : _('used in normal operation when eco mode is selected')
				});
				var notUsedField = Ext.create('DSS.component.myTemperatureControl', {
					targetMode : 'nominalTemperatureNotUsed',
					label : _('NotUsed Mode'),
                    description : _('used if romm is not used')
				});
				var nightField = Ext.create('DSS.component.myTemperatureControl', {
					targetMode : 'nominalTemperatureNight',
					label : _('Night Mode'),
                    description : _('used during planed night mode')
				});
				var holidayField = Ext.create('DSS.component.myTemperatureControl', {
					targetMode : 'nominalTemperatureHoliday',
					label : _('Holiday Mode'),
                    description : _('used during defined holidays in Presence Simulation')
				});

				var me = this;
				var oConfigTemperature = {
					defaultSortCol : 'zoneID',
					inactiveField : 'inactiveMode',
                    flex:1,
					storeConfig : {
						storeId : 'temperatureStore',
						fields : [ {
							name : 'zoneID'
						}, {
							name : 'zoneName'
                        }, {
							name : 'temperature'
						}, {
							name : 'effectiveNominalTemperature'
						}, {
							name : 'nominalTemperatureComfort'
						}, {
							name : 'nominalTemperatureEco'
						}, {
							name : 'nominalTemperatureNotUsed'
						}, {
							name : 'nominalTemperatureNight'
						}, {
							name : 'nominalTemperatureHoliday'
						}, {
							name : 'activeMode'
						}, {
							name : 'inactiveMode'
						}, {
							name : 'controlMode'
						}, {
							name : 'operationMode'
						} ]
					},
					columns : [ {
						header : _('zone'),
						dataIndex : 'zoneID',
						xtype : 'textcolumn',
						flex : 1,
						customColumnRenderer : function(val, metadata, record, rowIdx, colIdx, store, view) {
							var sText = dss.staticDataModel.getNameForZone(val);
							return formatCell(sText, record.get('activeMode'));
						}
					}, {
						header : _('current Temperature'),
						dataIndex : 'temperature',
						xtype : 'textcolumn',
						flex : 1,
						customColumnRenderer : function(val, metadata, record, rowIdx, colIdx, store, view) {
							var sText = me.formatTemperature(val, 1);
							if (val == null)
								sText = '-';
							if (val == "")
								sText = '-';
							return formatCell(sText, record.get('activeMode'));
						}
					}, {
						header : _('comfort mode temperature'),
						dataIndex : 'nominalTemperatureComfort',
						flex : 1,
						xtype : 'textcolumn',
						customColumnRenderer : function(val, metadata, record, rowIdx, colIdx, store, view) {
							var sText = me.formatTemperature(val, record.get('controlMode'));
							return formatCell(sText, record.get('activeMode'));
						}

					}, {
						header : _('eco mode temperature'),
						dataIndex : 'nominalTemperatureEco',
						flex : 1,
						xtype : 'textcolumn',
						customColumnRenderer : function(val, metadata, record, rowIdx, colIdx, store, view) {
							var sText = me.formatTemperature(val, record.get('controlMode'));
							return formatCell(sText, record.get('activeMode'));
						}

					}, {
						header : _('not used mode temperature'),
						dataIndex : 'nominalTemperatureNotUsed',
						flex : 1,
						xtype : 'textcolumn',
						customColumnRenderer : function(val, metadata, record, rowIdx, colIdx, store, view) {
							var sText = me.formatTemperature(val, record.get('controlMode'));
							return formatCell(sText, record.get('activeMode'));
						}

					}, {
						header : _('night mode temperature'),
						dataIndex : 'nominalTemperatureNight',
						flex : 1,
						xtype : 'textcolumn',
						customColumnRenderer : function(val, metadata, record, rowIdx, colIdx, store, view) {
							var sText = me.formatTemperature(val, record.get('controlMode'));
							return formatCell(sText, record.get('activeMode'));
						}

					}, {
						header : _('holiday mode temperature'),
						dataIndex : 'nominalTemperatureHoliday',
						flex : 1,
						xtype : 'textcolumn',
						customColumnRenderer : function(val, metadata, record, rowIdx, colIdx, store, view) {
							var sText = me.formatTemperature(val, record.get('controlMode'));
							return formatCell(sText, record.get('activeMode'));
						}

					} ],
					buttonText : {
						editButton : _('Edit Temperature settings')
					},
					editorConfig : {
						title : _('Temperaturesettings'),
						staticControls : [ {
							type : 'hidden',
							label : _('id'),
							id : 'zoneID'
						}, {
							type : 'hidden',
							label : _('id'),
							id : 'controlMode'
						}, {
							type : 'custom',
                            control : roomNameField,
							id : 'zoneName'
						}, {
							type : 'custom',
							control : comfortField,
							id : 'nominalTemperatureComfort'
						}, {
							type : 'custom',
							control : ecoField,
							id : 'nominalTemperatureEco'
						}, {
							type : 'custom',
							control : notUsedField,
							id : 'nominalTemperatureNotUsed'
						}, {
							type : 'custom',
							control : nightField,
							id : 'nominalTemperatureNight'
						}, {
							type : 'custom',
							control : holidayField,
							id : 'nominalTemperatureHoliday'
						} ]
					},
					listeners : {
						'loadDataStore' : function(grid) {
                            if (me.loadTemperatureParameter(grid)) {
                                grid.setStatus(_('Ready'));
                            } else {
                                grid.setStatus(_('Error'));
                            }
						},
						'saveEntry' : function(grid, values) {
							me.saveTemperaturParameter(values);
							me.reloadTemperatureSettings(grid, values);
                            me.loadStatusPanel();
						},
						'enableEntry' : function(grid, entry) {
							me.waitAndReloadTemperature(grid);
						},
						'disableEntry' : function(grid, entry) {
							me.waitAndReloadTemperature(grid);
						},
						'deleteEntry' : function(grid, entry) {
						},
						'testEntry' : function(grid, entry) {
						},
						'viewready' : function() {
							me.setNightModeVisibility(me.generalSettingsControl.ckbCentralHeatingNightmode.getValue());
						}
					}

				};

				this.temperaturePanel = Ext.create('DSS.gridPanel', oConfigTemperature);
                this.tempPanelIntro = Ext.create('Ext.form.Panel', {
                    border : 0,
                    width : 800,
                    height : 40,
                    labelWidth: 800,
                    padding : '10 0 10 5', // top right bottom left
                    html : '<span style="color:#005138;">' + _("Define here the nominal temperature and control value for the different operation modes") + '</span>'
                });
			},
			savePreheatingTime : function() {
                this.generalSettingsControl.preHeatingTimer = null;

                var preheatingTime = this.generalSettingsControl.preHeatingHoliday.getValue();
				dss.raiseEvent('heating-controller.general-settings', {
					actions : 'setPreheatingTime',
					value : preheatingTime
				});
				this.waitAndReloadGeneralSettings();
			},
			makeGeneralPanel : function() {
				var me = this;
				this.generalSettingsControl.radioSingleMultihouse = Ext.create('Ext.form.RadioGroup', {
					labelWidth : 230,
					width : 700,
					fieldLabel : _('Type of building'),
					items : [ {
						boxLabel : _('Singleapartmenthouse'),
						name : 'singleApartment',
						inputValue : 1,
						style : {
							padding : '10px',
							color : "#005138"
						}
					}, {
						boxLabel : _('Multiapartmenthouse'),
						name : 'singleApartment',
						inputValue : 2,
						style : {
							padding : '10px',
							color : "#005138"
						}
					} ],
					listeners : {
						change : function(oThis, newValue) {
							dss.raiseEvent('heating-controller.general-settings', {
								actions : 'setApartmentType',
								value : newValue.singleApartment
							});
							me.waitAndReloadGeneralSettings();
						}
					}
				});
				this.generalSettingsControl.ckbCentralHeatingNightmode = Ext.create('Ext.form.Checkbox', {
					labelWidth : 230,
					width : 700,
					fieldLabel : _('Heatingsystem'),
					boxLabel : _('The central heating has a own Nightmodehandling'),
					style : {
						padding : '10px',
						color : "#005138"
					},
					listeners : {
						change : function(oThis, newValue) {
							me.setNightModeVisibility(newValue);

							dss.raiseEvent('heating-controller.general-settings', {
								actions : 'setCentralHeatingNightModeActive',
								value : newValue
							});
							me.waitAndReloadGeneralSettings();
						}
					}
				});
				this.generalSettingsControl.radioHeatingAppGeneralSwitch = Ext.create('Ext.form.RadioGroup', {
					labelWidth : 230,
					width : 700,
					fieldLabel : _('General Heating Controller'),
					items : [ {
						boxLabel : _('Heating is enabled'),
						name : 'centralSwitch',
						inputValue : true,
						style : {
							padding : '10px',
							color : "#005138"
						}
					}, {
						boxLabel : _('Heating is disabled'),
						name : 'centralSwitch',
						inputValue : false,
						style : {
							padding : '10px',
							color : "#005138"
						}
					} ],
					listeners : {
						change : function(oThis, newValue) {
							dss.raiseEvent('heating-controller.general-settings', {
								actions : 'setHeatingEnabled',
								value : newValue.centralSwitch
							});
							me.waitAndReloadGeneralSettings();
						}
					}
				});
                this.generalSettingsControl.preHeatingTimer = null;
				this.generalSettingsControl.preHeatingHoliday = Ext.create('DSS.component.TimeSpanControl', {
					labelWidth : 230,
					width : 700,
					hideSecond : true,
					margin:'3 0 0 0',
                    hourMaxValue : 72,
					fieldLabel : _('Preheating Time'),
					listeners : {
						change : function(oThis, newValue) {
                            if (me.generalSettingsControl.preHeatingTimer !== null) {
                                clearTimeout(me.generalSettingsControl.preHeatingTimer); // complement
																							// to
																							// native
																							// setTimeout
																							// function
                            }

                            me.generalSettingsControl.preHeatingTimer = setTimeout(function(){me.savePreheatingTime();}, 1000);
						}
					}
				});
				
				this.generalSettingsControl.notUsedWhenAbsend = Ext.create('Ext.form.Checkbox', {
					labelWidth : 230,
					width : 700,
					fieldLabel : _('Behavior on absent'),
					boxLabel : _('Should set mode Not Used when absent'),
					style : {
						padding : '10px',
						color : "#005138"
					},
					listeners : {
						change : function(oThis, newValue) {
							dss.raiseEvent('heating-controller.general-settings', {
								actions : 'setUseNotUsedOnAbsent',
								value : newValue
							});
							me.waitAndReloadGeneralSettings();
						}
					}
				});

				this.generalSettingsControl.btnStartValveSettings = Ext.create('Ext.button.Button',{
					text:_('valve protection settings'),
					visible:false,
					handler: function () {
						var editor = Ext.create('DSS.addon.ValveEditor', {});
						editor.show();
					}
				});
				
				if (this.heatingStore_controllerMode == null) {
					this.heatingStore_controllerMode = Ext.create('Ext.data.Store', {
						storeId : 'heatingStore_controllerMode',
						fields : [ 'id', 'mode', 'name', 'object' ],
						proxy : {
							type : 'memory',
							reader : {
								type : 'json',
								root : 'items'
							}
						}
					});
				}
				if (this.heatingStore_referenceZone == null) {
					this.heatingStore_referenceZone = Ext.create('Ext.data.Store', {
						storeId : 'heatingStore_referenceZone',
						fields : [ 'id', 'name' ],
						proxy : {
							type : 'memory',
							reader : {
								type : 'json',
								root : 'items'
							}
						}
					});
				}
				for ( var iIndex = 0; iIndex < this.usableZones.length; iIndex++) {
					var oObject = this.usableZones[iIndex];
					if (oObject.ZoneID != 0) {
						this.heatingStore_referenceZone.add({
							id : '' + oObject.ZoneID,
							name : oObject.name
						});
					}
				}

				var ctrlOffset = Ext.create('Ext.form.field.Number', {
					fieldLabel : _('Offset'),
					labelWidth : 210,
					minValue : -100,
					maxValue : 300,
					value : 0,
					step : 1,
					decimalPrecision : 0,
					width : 300,
                    id : 'offsetNumber'
				});
				var ctrlEmergenyValue = Ext.create('Ext.form.field.Number', {
					fieldLabel : _('Emergency Value'),
					labelWidth : 210,
					minValue : 0,
					maxValue : 100,
					value : 0,
					step : 1,
					decimalPrecision : 0,
					width : 300,
                    id : 'emergencyNumber'
				});

				var ctrlEmergenyPanel = Ext.create('Ext.panel.Panel', {
					border : false,
					layout : 'hbox',
					height : 24,
					width : 600,
					items : [ ctrlEmergenyValue, {
						xtype : 'panel',
						height : 24,
						width : 400,
						margin : '3 0 0 5',
						border : false,
						html : '<span style="color:#005138">' + _('%') + '</span>'
					} ]
				});
				ctrlEmergenyPanel.setValue = function(val) {
					ctrlEmergenyValue.setValue(val);
				};
				ctrlEmergenyPanel.getValue = function() {
					return ctrlEmergenyValue.getValue();
				};
				ctrlEmergenyPanel.resetValue = function() {
					ctrlEmergenyValue.resetValue();
				};
				ctrlEmergenyPanel.isValid = function() {
					return ctrlEmergenyValue.isValid();
				};

				var ctrlPanel = Ext.create('Ext.panel.Panel', {
					border : false,
					layout : 'hbox',
					height : 24,
					width : 600,
					items : [ ctrlOffset, {
						xtype : 'panel',
						height : 24,
						width : 400,
						margin : '3 0 0 5',
						border : false,
						html : '<span style="color:#005138">'+_('% on the valve settings from the referenced zone') + '</span>'
					} ]
				});
				ctrlPanel.setValue = function(val) {
					ctrlOffset.setValue(val);
				};
				ctrlPanel.getValue = function() {
					return ctrlOffset.getValue();
				};
				ctrlPanel.resetValue = function() {
					ctrlOffset.resetValue();
				};
				ctrlPanel.isValid = function() {
					return ctrlOffset.isValid();
				};
                
                var ctrlAutoselectHelp = Ext.create('Ext.panel.Panel', {
					border : false,
					layout : 'hbox',
					height : 48,
					width : 700,
					items : [ {
						xtype : 'panel',
						height : 48,
						width : 600,
						margin : '3 0 0 5',
						border : false,
						html : '<span style="color:#005138">' + _('The operation mode is automatically determined by the connected device.') +  '<br>' + _('This is defined for each actuator under hardware / device properties.') + '</span>'
					} ]
                });
                ctrlAutoselectHelp.setValue = function(val) {};
				ctrlAutoselectHelp.getValue = function() {return null;};
				ctrlAutoselectHelp.resetValue = function() {};

                ctrlAutoselectHelp.isValid = function() {
                    return true;
                };
                
                var roomNameField = Ext.create('Ext.form.field.Display', {
                            border : 0,
                            height:18,
                            fieldStyle: {'fontWeight' : 'bold'}  
                });
				var oConfigRoomMode = {
					defaultSortCol : 'zoneID',
					storeConfig : {
						storeId : 'zoneParameterStore',
						fields : [ {
							name : 'zoneID'
						}, {
							name : 'zoneName'
						}, {
							name : 'baseMode'
						}, {
							name : 'controlMode'
						}, {
							name : 'sensorID'
						}, {
							name : 'actors'
						}, {
							name : 'ctrlOffset'
						}, {
							name : 'referenceZone'
						}, {
							name : 'emergenyValue'
						}, {
                            name : 'parameter'
                        }]
					},
					columns : [ {
						header : _('zone'),
						dataIndex : 'zoneID',
						xtype : 'textcolumn',
						flex : 1,
						customColumnRenderer : function(val) {
							return dss.staticDataModel.getNameForZone(val);
						}
					}, {
						header : _('base mode'),
						dataIndex : 'baseMode',
						xtype : 'textcolumn',
						flex : 1,
						customColumnRenderer : function(val, metadata, record, rowIdx, colIdx, store, view) {
							if (val == "") {
								var oRecord = me.heatingStore_controllerMode.findRecord('mode', 0);
								return oRecord.get('name');
							} else {
/*
 * baseMode definitions: 0 Aus 1 .. n Template ID's -1 Custom PID, no cloud
 * template id -2 Zone Follower -3 Fixed -4 Manual -5 Automatic - PID -6
 * Automatic - Failed -7 Automatic - All valves are Unknown -8 Automatic -
 * no sensor
 */
								if (val == "-1")
									return _('custom PID-Controller');
								if (val == "-2")
									return Ext.String.format2(_('following {zonename} with offset {offset}'), {
										zonename : dss.staticDataModel.getNameForZone(record.get('referenceZone')),
										offset : record.get('ctrlOffset')
									});
								if (val == "-4") {
									return _('manual mode');
								}
                                if ((val == "-5") || (val == "-6") || (val == "-7") || (val == "-8")) {
	                                var returnString = _('automatic mode');
                                    var oRecord = me.heatingStore_controllerMode.findRecord('id', (record.get('parameter')+''));
                                    if (oRecord) {
                                        returnString += ' (';
                                        returnString += oRecord.get('name');
                                        returnString += ')';
                                    }
                                    return returnString;
								}
								var oRecord = me.heatingStore_controllerMode.findRecord('id', val);
								return oRecord.get('name');
							}
						}
					}, {
						header : _('Roomsensor defined'),
						dataIndex : 'sensorID',
						flex : 1,
						xtype : 'textcolumn',
						customColumnRenderer : function(val) {
							if (val != null) {
								var oDevice = dss.staticDataModel.getDevice(val);
                                var sString = '<table><tr><td style="padding-right:5px;"><img src="' + dss.staticDataModel.getImageForDevice(oDevice) + '"></td>';
                                sString += '<td' + (oDevice.present ? '' : ' style="color:grey;"') + '>' + oDevice.name + '</td></tr></table>';
								return sString;
							} else
								return _('no');
						}
					}, {
						header : _('Actors defined'),
						dataIndex : 'actors',
						flex : 1,
						xtype : 'textcolumn',
						customColumnRenderer : function(val) {
							if (val.length == 0)
								return _('no');
							var sString = '<table>';
							for ( var iIndex = 0; iIndex < val.length; iIndex++) {
								oDevice = val[iIndex];
								sString += '<tr><td style="padding-right:5px;"><img src="' + dss.staticDataModel.getImageForDevice(oDevice) + '"></td>';

								sString += '<td' + (oDevice.present ? '' : ' style="color:grey;"') + '>' + oDevice.name + '</td></tr>';
							}
							return sString + '</table>';
						}
					} ],
					buttonText : {
						editButton : _('Edit Roomsettings')
					},
					editorConfig : {
						title : _('Roomsettings'),
						staticControls : [ {
							type : 'custom',
                            control : roomNameField,
							id : 'zoneName'
						}, {
							type : 'combobox',
							label : _('Base Mode'),
							storeid : 'heatingStore_controllerMode',
							storedisplayid : 'name',
							storevalueid : 'id',
							labelWidth:210,
							id : 'baseMode'
						}, {
							type : 'combobox',
							label : _('referenced Zone'),
							storeid : 'heatingStore_referenceZone',
							storedisplayid : 'name',
							storevalueid : 'id',
							labelWidth:210,
							id : 'referenceZone'
						}, {
							type : 'custom',
							label : _('Offset'),
							id : 'ctrlOffset',
							control : ctrlPanel
						}, {
							type : 'custom',
							id : 'emergenyValue',
							control : ctrlEmergenyPanel
						}, {
							type : 'hidden',
							label : _('id'),
							id : 'zoneID'
						}, {
                            type : 'custom',
                            label : '',
                            id : 'ctrlAutoselectHelp',
                            control : ctrlAutoselectHelp
                            
                        }]
					},
					defaultNewEntryValues : {
						ctrlOffset : 0,
						emergenyValue : 75
					},
					listeners : {
						'loadDataStore' : function(grid) {
							me.loadRoomParameter(grid);
						},
						'saveEntry' : function(grid, values) {

							var controlerMode = values.baseMode;
							if (isNaN(controlerMode)) {
								return;
							}

							me.waitDialog = new Ext.LoadMask(Ext.getBody(), {
								msg : _('Please wait, changing controller mode ...'),
								indicator : false
							});
							me.waitDialog.show();

							var iZoneID = values.zoneID;
							var controlerMode = values.baseMode;

							var oRecord = me.heatingStore_controllerMode.findRecord('id', controlerMode);
							var oDataObject = oRecord.get('object');

							if (oDataObject.controllerMode == ControlModes.cControlModeOff /* 0 */) {
								dss.raiseEvent('heating-controller.controller-config', {
									actions : 'resetControllerMode',
									zoneID : iZoneID
								});
							}
							if (oDataObject.controllerMode == ControlModes.cControlModePID /* 1 */) {

								dss.raiseEvent('heating-controller.controller-config', {
									actions : 'setControllerModePID',
									zoneID : iZoneID,
									definitionID : controlerMode,
									emergency : values.emergenyValue

								});
							}
							if (oDataObject.controllerMode == ControlModes.cControlModeZoneFollower /* 2 */) {

								dss.raiseEvent('heating-controller.controller-config', {
									actions : 'setControllerModeZoneFollower',
									zoneID : iZoneID,
									followedZone : values.referenceZone,
									offset : values.ctrlOffset,
									emergency : values.emergenyValue
								});
							}
							if (oDataObject.controllerMode == ControlModes.cControlModeFixedValue /* 3 */) {
								dss.raiseEvent('heating-controller.controller-config', {
									actions : 'setControllerModeFixedValue',
									zoneID : iZoneID
								});
							}
							if (oDataObject.controllerMode == ControlModes.cControlModeAutomatic /* 5 */) {
								dss.raiseEvent('heating-controller.controller-config', {
									actions : 'setControllerModeAutomatic',
									zoneID : iZoneID,
									definitionID : -1,
									autoselect : 1,
									emergency : values.emergenyValue
								});
							}
							me.waitAndReloadRoomParameter(grid);
						},
						'enableEntry' : function(grid, entry) {
							me.waitAndReloadRoomParameter(grid);
						},
						'disableEntry' : function(grid, entry) {
							me.waitAndReloadRoomParameter(grid);
						}
					}

				};

				var roomPanel = Ext.create('DSS.gridPanel', oConfigRoomMode);
				var baseModeControl = roomPanel.editorWindow.editControls['baseMode'].addListener('change', function() {
					if (isNaN(this.getValue())) {
						return;
					}
					if (this.getValue() == -1) {
						this.referenceZoneID.setVisible(false);
                        this.ctrlOffset.getChildByElement('offsetNumber').allowBlank=true;
						this.ctrlOffset.setVisible(false);
                        this.emergenyValue.getChildByElement('emergencyNumber').allowBlank=false;
						this.emergenyValue.setVisible(true);
						this.emergenyValue.setValue(75);
						this.setValue(_("custom PID Controller, please choose a other controller"));
                        this.ctrlAutoselectHelp.setVisible(false);
						return;
					}
					if (this.getValue() == -4) {
						this.referenceZoneID.setVisible(false);
                        this.ctrlOffset.getChildByElement('offsetNumber').allowBlank=true;
						this.ctrlOffset.setVisible(false);
                        this.emergenyValue.getChildByElement('emergencyNumber').allowBlank=false;
						this.emergenyValue.setVisible(true);
						this.emergenyValue.setValue(75);
						this.setValue(_("manual Mode, please choose a other controller"));
                        this.ctrlAutoselectHelp.setVisible(false);
						return;
					}
					if (this.getValue() == -5) {
						this.referenceZoneID.setVisible(false);
                        this.ctrlOffset.getChildByElement('offsetNumber').allowBlank=true;
						this.ctrlOffset.setVisible(false);
                        this.emergenyValue.getChildByElement('emergencyNumber').allowBlank=false;
						this.emergenyValue.setVisible(true);
						this.emergenyValue.setValue(75);
// this.setValue(_("Automatic controller configuration"));
                        this.ctrlAutoselectHelp.setVisible(true);
						return;
					}
					if (this.getValue() == -6) {
						this.referenceZoneID.setVisible(false);
                        this.ctrlOffset.getChildByElement('offsetNumber').allowBlank=true;
						this.ctrlOffset.setVisible(false);
                        this.emergenyValue.getChildByElement('emergencyNumber').allowBlank=false;
						this.emergenyValue.setVisible(true);
                        var returnString = _('automatic mode');
                        var oRecord = HeatingControl.heatingStore_controllerMode.findRecord('id', 0);
                        if (oRecord) {
                            returnString += ' (';
                            returnString += oRecord.get('name');
                            returnString += ')';
                        }
  						this.setValue(returnString);
                        this.ctrlAutoselectHelp.setVisible(true);
						return;
					}
					if ((this.getValue() == -7) || (this.getValue() == -8)) {
						this.referenceZoneID.setVisible(false);
                        this.ctrlOffset.getChildByElement('offsetNumber').allowBlank=true;
						this.ctrlOffset.setVisible(false);
                        this.emergenyValue.getChildByElement('emergencyNumber').allowBlank=false;
						this.emergenyValue.setVisible(true);
                        var returnString = _('automatic mode');
                        var oRecord = HeatingControl.heatingStore_controllerMode.findRecord('id', 0);
                        if (oRecord) {
                            returnString += ' (';
                            returnString += oRecord.get('name');
                            returnString += ')';
                        }
  						this.setValue(returnString);
                        this.ctrlAutoselectHelp.setVisible(true);
						return;
					}
					var oModeType =  this.cboCombo.getStore().findRecord('id', this.getValue()).get('object').controllerMode;
                    this.referenceZoneID.cboCombo.allowBlank = true;
					if (oModeType == ControlModes.cControlModeZoneFollower) {
                        this.referenceZoneID.cboCombo.allowBlank = false;
						this.referenceZoneID.setVisible(true);
                        this.ctrlOffset.getChildByElement('offsetNumber').allowBlank=false;
						this.ctrlOffset.setVisible(true);
						if (this.ctrlOffset.getValue() == null)
							this.ctrlOffset.setValue(0);
						var selectedZone = this.zoneID.getValue();
                        this.referenceZoneID.cboCombo.getStore().clearFilter(true);
                        this.referenceZoneID.cboCombo.getStore().filter({
							filterFn : function(iItem) {
								if (('' + iItem.get('id')) == ('' + selectedZone))
									return false;
								if (me.zoneControlValues[iItem.get('id')] != null)
									if (me.zoneControlValues[iItem.get('id')].ControllerMode == ControlModes.cControlModeZoneFollower)
										return false;
								return true;
							}
						});
                        this.emergenyValue.getChildByElement('emergencyNumber').allowBlank=false;
						this.emergenyValue.setVisible(true);
						this.emergenyValue.setValue(75);
                        this.ctrlAutoselectHelp.setVisible(false);
					} else if (oModeType == ControlModes.cControlModeFixedValue) {
						this.referenceZoneID.setVisible(false);
                        this.ctrlOffset.getChildByElement('offsetNumber').allowBlank=true;
						this.ctrlOffset.setVisible(false);
                        this.emergenyValue.getChildByElement('emergencyNumber').allowBlank=true;
						this.emergenyValue.setVisible(false);	
                        this.ctrlAutoselectHelp.setVisible(false);
					} else {
						this.referenceZoneID.setVisible(false);
                        this.ctrlOffset.getChildByElement('offsetNumber').allowBlank=true;
						this.ctrlOffset.setVisible(false);
						if (oModeType == ControlModes.cControlModeOff) {
                            this.emergenyValue.getChildByElement('emergencyNumber').allowBlank=true;
    						this.emergenyValue.setVisible(false);
                        } else {
                            this.emergenyValue.getChildByElement('emergencyNumber').allowBlank=false;
							this.emergenyValue.setVisible(true);
							var baseObject=this.cboCombo.getStore().findRecord('id', this.getValue()).get('object');
							if (baseObject.emergency==null)
								this.emergenyValue.setValue(75);
							else
								this.emergenyValue.setValue(baseObject.emergency);
						}
                        if (oModeType == ControlModes.cControlModeAutomatic) {
                            this.ctrlAutoselectHelp.setVisible(true);
                        } else {
                            this.ctrlAutoselectHelp.setVisible(false);
                        }
					}

				});
				roomPanel.editorWindow.editControls['baseMode'].referenceZoneID = roomPanel.editorWindow.editControls['referenceZone'];
				roomPanel.editorWindow.editControls['baseMode'].zoneID = roomPanel.editorWindow.editControls['zoneID'];
				roomPanel.editorWindow.editControls['baseMode'].ctrlOffset = roomPanel.editorWindow.editControls['ctrlOffset'];
				roomPanel.editorWindow.editControls['baseMode'].emergenyValue = roomPanel.editorWindow.editControls['emergenyValue'];
				roomPanel.editorWindow.editControls['baseMode'].ctrlAutoselectHelp = roomPanel.editorWindow.editControls['ctrlAutoselectHelp'];
                
				var referenceZoneID = roomPanel.editorWindow.editControls['referenceZone'];
				var zoneID = roomPanel.editorWindow.editControls['zoneID'];
				this.generalPanel = Ext.create('Ext.panel.Panel', {
					border : false,
					flex : 1,
					style : {
						padding : '10px'
					},
					layout : {
						type : 'vbox',
						align : 'left'
					},
					items : [ {
						flex : 0,
						height : 180,
						width : "100%",
						xtype : 'panel',
						border : false,
						items : [ {
							xtype : 'fieldset',
							style : {
								padding : '10px'
							},
							height : 174,
							title : _('General Settings'),
							items : [ {
								xtype : 'panel',
								layout : 'hbox',
								border : false,
								items : [ this.generalSettingsControl.radioHeatingAppGeneralSwitch ]
							}, {
								xtype : 'panel',
								layout : 'hbox',
								border : false,
								items : [ this.generalSettingsControl.radioSingleMultihouse ]
							},this.generalSettingsControl.ckbCentralHeatingNightmode,
							{
								xtype : 'panel',
								layout : 'hbox',
								border : false,
								margin : '0 0 6 0', 
								items : [  this.generalSettingsControl.notUsedWhenAbsend /* this.generalSettingsControl.btnStartValveSettings */ ]
							},
							this.generalSettingsControl.preHeatingHoliday ]
						} ]
					}, {
						flex : 1,
						xtype : 'fieldset',
						style : {
							padding : '10px'
						},
						layout : 'fit',
						reserveScrollbar : true,
						width : "100%",
						title : _('Roomsettings'),
						items : [ roomPanel ]
					} ]
				});
			},
			statusControl : null,
            setStartPanelStatus : function (text) {
                if (this.startPanelStatusText) {
                    this.startPanelStatusText.setText(text);
                }
            },
			makeStatusPanel : function() {
				var me = this;
				if (this.statusControl == null) {
					this.statusControl = {};
					this.statusControl.radioApartmentMode = Ext.create('Ext.form.RadioGroup', {
						labelWidth : 230,
						width : 700,
						fieldLabel : _('actual Apartment Mode'),
						items : [ {
							boxLabel : _('Comfort Mode'),
							name : 'apartmentMode',
							inputValue : OperationModes.Comfort,
							style : {
								padding : '10px',
								color : "#005138"
							}
						}, {
							boxLabel : _('Eco Mode'),
							name : 'apartmentMode',
							inputValue : OperationModes.Eco,
							style : {
								padding : '10px',
								color : "#005138"
							}
						} /*
							 * , { boxLabel : _('Night Mode'), name :
							 * 'apartmentMode', inputValue :
							 * OperationModes.Night, style : { padding : '10px',
							 * color : "#005138" } }
							 */],
						listeners : {
							change : function(oThis, newValue, oldValue) {

								me.waitDialog = new Ext.LoadMask(Ext.getBody(), {
									msg : _('Please wait, changing operation mode ...'),
									indicator : false
								});
								me.waitDialog.show();


								dss.raiseEvent('heating-controller.operation-mode', {
									actions : 'setOperationMode',
									zoneID : 0,
									operationMode : newValue.apartmentMode
								});

								var iSubscriptID = Math.floor(Math.random() * 100000000000);
								dss.ajaxSyncRequest('/json/event/subscribe', {
									subscriptionID : iSubscriptID,
									name : 'heating-controller.answer'
								});
								dss.ajaxAssyncRequestWithoutAnswer('/json/event/get', {
									subscriptionID : iSubscriptID,
									timeout : 60 * 1000
								}, function() {
									dss.ajaxAssyncRequestWithoutAnswer('/json/event/unsubscribe', {
										subscriptionID : iSubscriptID,
										name : 'heating-controller.answer'
									});
									setTimeout(function() {me.loadStatusPanel();}, 1000 /*
																						 * 1
																						 * sec
																						 */);
								});

							}
						}
					});
					
				}

				var oConfigOverviewMode = {
					defaultSortCol : 'zoneID',
					storeConfig : {
						storeId : 'overviewStore',
						fields : [ {
							name : 'zoneID'
						}, {
							name : 'actualTemperature'
						}, {
							name : 'targetTemperature'
						}, {
							name : 'actualMode'
						}, {
							name : 'disabled'
						}, {
							name : 'type'							
						}, {
							name : 'coolModeAvaible'
						}, {
                            name : 'emergencyMode'
                        }]
					},
					disabledProperty:'disabled',
                    showStatusBar : false,
                    inactiveEnablerField: 'coolModeAvaible',
					columns : [ {
						header : _('zone'),
						dataIndex : 'zoneID',
						xtype : 'textcolumn',
						flex : 1,
						customColumnRenderer : function(val) {
							return dss.staticDataModel.getNameForZone(val);
						}
					}, {
						header : _('actual Temperature'),
						dataIndex : 'actualTemperature',
						xtype : 'textcolumn',
						flex : 1,
						customColumnRenderer : function(val, metadata, record, rowIdx, colIdx, store, view) {
							var sText = me.formatTemperature(val, record.get('type'));
							if (val == null)
								sText = '-';
							if (val == "")
								sText = '-';
                            if (record.get('emergencyMode')===true) {
                                sText = '-'
                            }
							return sText;

						}
					}, {
						header : _('target Temperature'),
						dataIndex : 'targetTemperature',
						flex : 1,
						xtype : 'textcolumn',
						customColumnRenderer : function(val, metadata, record) {
							var sText = me.formatTemperature(val, record.get('type'));
							if (val == null)
								sText = '-';
							if (val == "")
								sText = '-';
                            if (record.get('emergencyMode')===true) {
                                sText = '-'
                            }
							return sText;
						}
					}, {
						header : _('actual Mode'),
						dataIndex : 'actualMode',
						flex : 1,
						xtype : 'textcolumn',
						customColumnRenderer : function(val, metadata, record) {
                            if (record.get('emergencyMode')===true) {
                               return _('emergency state');
                            } else {
                                switch (val) {
                                case OperationModes.Off:
                                    return _('Off Mode');
                                    break;
                                case OperationModes.Comfort:
                                    return _('Comfort Mode');
                                    break;
                                case OperationModes.Eco:
                                    return _('Eco Mode');
                                    break;
                                case OperationModes.NotUsed:
                                    return _('NotUsed Mode');
                                    break;
                                case OperationModes.Night:
                                    return _('Night Mode');
                                    break;
                                case OperationModes.Holiday:
                                    return _('Holiday Mode');
                                    break;
                                }
                                return '-';
                            }
						}
					}, {
						header : _('NotUsed Mode'),
						dataIndex : 'disabled',
						resizable : false,
						xtype : 'disabledcolumn',
						width : 60
					} ],
					buttonText : {
						/*
						 * enableButton : _('Set Not Used for zone'),
						 * disableButton : _('Remove Not Used for zone')
						 */
					},
					editorConfig : {
						title : '',
						staticControls : [ {
							type : 'hidden',
							label : _('id'),
							id : 'zoneID'
						} ]
					},
					listeners : {
						'loadDataStore' : function(grid) {
							grid.setDisabled(true);
							me.setStartPanelStatus(_('changing Mode'));
							me.zoneControlValues=null;
							if (me.loadStatusParameter(grid)) {
                                me.setStartPanelStatus(_('Ready'));
                            } else {
                                me.setStartPanelStatus(_('Error'));
                            }                            
						},
						'saveEntry' : function(grid, values) {
						},
						'enableEntry' : function(grid, entry) {
							grid.setDisabled(true);
							me.setStartPanelStatus(_('changing Mode'));
							var zoneID = entry.get('zoneID');
							dss.raiseEvent('heating-controller.operation-mode', {
								actions : 'setOperationMode',
								zoneID : zoneID,
								operationMode : OperationModes.NotUsed
							});
							me.waitAndReloadStatus(grid);
						},
						'disableEntry' : function(grid, entry) {
							grid.setDisabled(true);
							me.setStartPanelStatus(_('changing Mode'));
							var zoneID = entry.get('zoneID');
							dss.raiseEvent('heating-controller.operation-mode', {
								actions : 'resetOperationMode',
								zoneID : zoneID,
								operationMode : OperationModes.NotUsed
							});
							me.waitAndReloadStatus(grid);
						}
					}

				};

				this.overviewPanel = Ext.create('DSS.gridPanel', oConfigOverviewMode);

				this.sensorPanel = Ext.create('Ext.panel.Panel', {
					border : false
				});
                this.startPanelStatusText = Ext.create('Ext.toolbar.TextItem',{});
                this.startPanelStatusBar = Ext.create('Ext.toolbar.Toolbar', {
                    width : '100%',
                    items:[this.startPanelStatusText,
                    '->',
                        {
                            text:_('Refresh'),
                            handler: function(){
                                me.setStartPanelStatus(_('Refreshing...'));
                                me.loadStatusPanel();
                            },
                            icon: dss.staticDataModel.baseImagePath + 'buttonIcons/refresh.png'
            			}]
                });

				this.startPanel = Ext.create('Ext.panel.Panel', {
					border : false,
					flex : 1,
					style : {
						padding : '10px'
					},
					layout : {
						type : 'vbox',
						align : 'left'
					},
					items : [ {
						height : 70,
						width : "100%",
						xtype : 'panel',
						border : false,
						items : [ {
							xtype : 'fieldset',
							style : {
								padding : '10px'
							},
							height : 70,
							title : _('Apartment'),
							items : [ this.statusControl.radioApartmentMode ]
						} ]
					}, {
						flex : 3,
						xtype : 'fieldset',
						style : {
							padding : '10px'
						},
						layout : 'fit',
						reserveScrollbar : true,
						width : "100%",
						title : _('Zones'),
						items : [ this.overviewPanel ]

					}, {
						flex : 1,
						xtype : 'panel',
						width : "100%",
						border : false,
						layout : 'fit',
						items : [ {
							xtype : 'fieldset',
							style : {
								padding : '10px'
							},
							flex : 1,
							layout : 'auto',
							height : "100%",
							title : _('Messages'),
							autoScroll : true,
							items : [ {
								xtype : 'panel',
								id : 'sensorStatusPanel',
								border : false
							} ]
						} ]
					}, 
                    this.startPanelStatusBar 
                ]
				});
				this.loadStatusPanel(true);
			},
			waitAndReloadStatus:function (grid) {
				var me = this;
				var iSubscriptID = Math.floor(Math.random() * 100000000000);
				dss.ajaxSyncRequest('/json/event/subscribe', {
					subscriptionID : iSubscriptID,
					name : 'heating-controller.answer'
				});
				dss.ajaxAssyncRequestWithoutAnswer('/json/event/get', {
					subscriptionID : iSubscriptID,
					timeout : 60 * 1000
				}, function() {
					me.zoneControlValues=null;
                    if (me.loadStatusParameter(grid)) {
                        me.setStartPanelStatus(_('Ready'));
                    } else {
                        me.setStartPanelStatus(_('Error'));
                    }
					dss.ajaxAssyncRequestWithoutAnswer('/json/event/unsubscribe', {
						subscriptionID : iSubscriptID,
						name : 'heating-controller.answer'
					});
					
				});
			},
			loadStatusParameter : function(grid) {
				var success = this.fetchZoneControlValues();
				var fApartmentEco = dss.ajaxSyncRequest('/json/property/query2', {
					'query' : '/usr/addon-states/heating-controller/*(name,value)'
				});
				var result = Ext.JSON.decode(fApartmentEco);
				var values = [];
                var messages=[];
				for ( var iIndex = 0; iIndex < this.possibleZones.length; iIndex++) {
					var sZoneID = this.possibleZones[iIndex];
					var oParameter = {
						zoneID : sZoneID,
						coolModeAvaible: true,
                        emergencyMode: false
					};
					if (this.zoneControlValues[sZoneID].IsConfigured) {
						if (result.ok) {
							if (result.result['operationmode.' + sZoneID + '.NotUsed'] != null) {
								if (result.result['operationmode.' + sZoneID + '.NotUsed'].value != null) {
									oParameter.disabled = (result.result['operationmode.' + sZoneID + '.NotUsed'].value==2);
								}

							}
                        } else {
                           success = false;
                        }
                        if (this.zoneControlValues[sZoneID].ControlState == 3) {
                            var sEmergencyValue = '(' +  this.zoneControlValues[sZoneID].config.EmergencyValue + '%)';
                            //set error to messages
                            var msg = Ext.String.format2(_('Controller in {zonename} is in emergency state {emergencyvalue}'), {
                                            zonename : dss.staticDataModel.getNameForZone(sZoneID),
                                            emergencyvalue : sEmergencyValue});
                            messages.push(msg);
                            oParameter.emergencyMode = true;

                        }
					}
					if (this.zoneControlValues[sZoneID].actualTemperature != null)
						oParameter.actualTemperature = parseFloat(this.zoneControlValues[sZoneID].actualTemperature);
					if (this.zoneControlValues[sZoneID].NominalValue != null) {
						oParameter.type = this.zoneControlValues[sZoneID].ControlMode;
						oParameter.actualTemperature = this.zoneControlValues[sZoneID].TemperatureValue;
						oParameter.targetTemperature = this.zoneControlValues[sZoneID].NominalValue;
					}
                    if (this.zoneControlValues[sZoneID].OperationMode!=null) {
                        oParameter.actualMode = this.zoneControlValues[sZoneID].OperationMode;
                        //if control mode not 'off' and we are not in emergency mode set the cool mode enabler field visible
                        if ((oParameter.actualMode != ControlModes.cControlModeOff) && (oParameter.emergencyMode === false)) {
                            oParameter.coolModeAvaible=false;
                        }
                    }
                    var oDevices = dss.staticDataModel.getDevicesForZone(sZoneID, 48);
                    oParameter.actors = [];
                    for ( var jIndex = 0; jIndex < oDevices.length; jIndex++) {
                        if (oDevices[jIndex].outputMode > 0)
                            oParameter.actors.push(oDevices[jIndex]);
                    }
                    
                    if ((this.zoneControlValues[sZoneID].IsConfigured) && (oParameter.actors.length>0))
                    	values.push(oParameter);
                    else {
                    	if (this.zoneControlValues[sZoneID].actualTemperature!=null)
	                    	values.push(oParameter);
                    }
				}
				
				grid.loadParameter(values);
                this.updateSensorStatus(messages);
				grid.setDisabled(false);
                return success;
			},
			loadStatusPanel : function(fFirstLoad) {
                var success = true;
				// this.setStartPanelStatus(_('Refreshing...'))
				if (fFirstLoad == null)
					success = this.fetchZoneControlValues(true);
				if (fFirstLoad == false)
					success = this.fetchZoneControlValues(true);
				
				var fApartmentEco = dss.ajaxSyncRequest('/json/property/query2', {
					'query' : '/usr/addon-states/heating-controller/operationmode.0.Comfort(value)'
				});
				var result = Ext.JSON.decode(fApartmentEco);

				if (result.ok) {
					this.statusControl.radioApartmentMode.suspendEvents();
					this.statusControl.radioApartmentMode.setValue(null);
					try {
						if (result.result['operationmode.0.Comfort'].value==2) {
							this.statusControl.radioApartmentMode.setValue({
								'apartmentMode' : OperationModes.Eco
							});
						} else {
							this.statusControl.radioApartmentMode.setValue({
								'apartmentMode' : OperationModes.Comfort
							});
						}
					} catch (e) {
					}
					this.statusControl.radioApartmentMode.resumeEvents();
				} else {
                    success = false;
                }
				
				if (fFirstLoad == null) {
					success = success && this.loadStatusParameter(this.overviewPanel);
                } else {
					if (fFirstLoad == false) {
						success = success && this.loadStatusParameter(this.overviewPanel);
                    } else {
                        this.updateSensorStatus([]);
                    }
                }
				
				if (this.waitDialog != null) {
					this.waitDialog.hide();
					this.waitDialog = null;
				}
                if (success) {
                    this.setStartPanelStatus(_('Ready'));
                } else {
                    this.setStartPanelStatus(_('Error'));
                }
			},
            updateSensorStatus : function (errMsgs) {
                var message=[];
				var sHTML='';
                var sGeneralErrorHTML='';
                //remote connectivity
                var fCloud=dss.isCloudAvaible(true);
				if (fCloud==false) {
					message.push(_('The remote connectivity is not configurated, so there will be some limitation when using the heating system via digitalSTROM'));
				}
                //general settings
                if (!this.controllerDefined) {
					message.push(_('Set the controller mode either manual or in the parameter of the device.'));                    
                }
				for (var iIndex=0;iIndex<message.length;iIndex++) {
					sGeneralErrorHTML+='<li style="color:#005138;list-style: disc outside none;">' + message[iIndex] + '</li>';
				}
                if (sGeneralErrorHTML != '') {
					sGeneralErrorHTML='<ul style="padding-left: 20px; margin-top: 0px; margin-bottom: 0px;">' + sGeneralErrorHTML + '</ul>';
                }
                //sensor error states
                var sSensorErrorHTML='';
                for (var iIndex=0;iIndex<errMsgs.length;iIndex++) {
					sSensorErrorHTML += '<li style="color:#005138;list-style: disc outside none;">' + errMsgs[iIndex] + '</li>';
				}
                if (sSensorErrorHTML != '') {
					sSensorErrorHTML='<ul style="padding-left: 20px; margin-top: 0px; margin-bottom: 0px;">' + sSensorErrorHTML + '</ul>';
                }
                //auto select errors
                var sAutoSelectHTML='';
                for (var jIndex = 0; jIndex < this.autoSelectMessage.length; jIndex++) {
                    sAutoSelectHTML += '<li style="color:#005138;list-style: disc outside none;">' +this.autoSelectMessage[jIndex] + '</li>';
                }
                if (sAutoSelectHTML != '') {                     
                    //sAutoSelectHTML += '<span style="color:#005138">' +  _('Please define the connected device for each actor under hardware / device properties or choose the operation mode manually') + '</span>';;
                    sAutoSelectHTML = '<ul style="padding-left: 20px; margin-top: 0px; margin-bottom: 0px;">' + sAutoSelectHTML + '</ul>';
                }

                //load error
                message = [];
                if (this.loadError.errorPresent) {
                    var sMessage = _('Error') + ': ' + this.loadError.errorDescr;
                    message.push(sMessage);
                }
                var sLoadErrorHTML = '';
                if (message.length > 0) {
                    sLoadErrorHTML = '<ul style="padding-left: 20px; margin-top: 0px; margin-bottom: 0px;">';
                    for (var iIndex=0;iIndex<message.length;iIndex++) {
                        sLoadErrorHTML+='<li style="color:#005138;list-style: disc outside none;">' + message[iIndex] + '</li>';
                    }
                    sLoadErrorHTML += '</ul>';
				}
                //now put it all together
                if (sGeneralErrorHTML != '') {
                    if (sHTML === '') {
                        sHTML += '<li style="color:#005138; list-style:none;">' + sGeneralErrorHTML + '</li>';
                    } else {
                        sHTML += '<li style="color:#005138; list-style:none;">&nbsp' + sGeneralErrorHTML + '</li>';                        
                    }
                }
                if (sSensorErrorHTML != '') {
                    if (sHTML === '') {
                        sHTML += '<li style="color:#005138; list-style:none;">' + sSensorErrorHTML + '</li>';
                    } else {
                        sHTML += '<li style="color:#005138; list-style:none;">&nbsp' + sSensorErrorHTML + '</li>';
                    }
                }
                    
                if (sAutoSelectHTML != '') {
                    if (sHTML === '') {
                        sHTML += '<li style="color:#005138; list-style:none;">' + sAutoSelectHTML + '</li>';
                    } else {
                        sHTML += '<li style="color:#005138; list-style:none;">&nbsp' + sAutoSelectHTML + '</li>';
                    }
                }
                if (sLoadErrorHTML != '') {
                    if (sHTML === '') {
                        sHTML += '<li style="color:#005138; list-style:none;">' + sLoadErrorHTML + '</li>';
                    } else {
                        sHTML += '<li style="color:#005138; list-style:none;">&nbsp' + sLoadErrorHTML + '</li>';
                    }
                }
				if (sHTML=='') {
					sHTML = '<span style="color:#005138; list-style:none;">&nbsp' + _("No Messages") + '</span>';
                } else {
                    sHTML = '<ul style="padding-left: 20px; margin-top: 0px; margin-bottom: 0px;">' + sHTML + '</ul>';
                }
                
				var item = Ext.create('Ext.panel.Panel', {
					border : false,
                    overflowY: 'auto',
					html : sHTML
				});
                if (Ext.getCmp('sensorStatusPanel')) {
                    Ext.getCmp('sensorStatusPanel').removeAll();
                    Ext.getCmp('sensorStatusPanel').add(item);
                }
            },
			waitDialog : null,
			getContent : function() {
				var me = this;
                this.fetchZoneControlValues();
				this.makeStatusPanel();
				this.makeGeneralPanel();
				this.makeTemperaturePanel();
				this.makeTimePanel();
				this.startTab = Ext.create('Ext.panel.Panel', {
					title : _('Overview'),
					layout : 'fit',
					items : [ this.startPanel ]
				});
				this.temperatureTab = Ext.create('Ext.panel.Panel', {
					title : _('Temperature and Offsets'),
                    layout : {
						type : 'vbox',
						align : 'stretch'
					},
					items : [this.tempPanelIntro, this.temperaturePanel]
				});
				this.timeTab = Ext.create('Ext.panel.Panel', {
					layout : 'fit',
					title : _('Time profiles'),
					items : [ this.timePanel ]
				});
				this.generalTab = Ext.create('Ext.panel.Panel', {
					layout : 'fit',
					title : _('General Settings'),
					items : [ this.generalPanel ]
				});
				this.tabControl = Ext.create('Ext.tab.Panel', {
					layout : 'fit',
					cls : 'generalTab',
					items : [ this.startTab, this.temperatureTab, this.timeTab, this.generalTab ]
				});

				return this.tabControl;
			},
			waitAndReloadGeneralSettings : function() {
				var me = this;
				var iSubscriptID = Math.floor(Math.random() * 100000000000);
				dss.ajaxSyncRequest('/json/event/subscribe', {
					subscriptionID : iSubscriptID,
					name : 'heating-controller.answer'
				});
				dss.ajaxAssyncRequestWithoutAnswer('/json/event/get', {
					subscriptionID : iSubscriptID,
					timeout : 60 * 1000
				}, function() {
					me.loadGeneralSettings();
					me.loadTimedSettings();
					me.updateTimeStore();
					if (HeatingControl != undefined) {
						tabActivation(!me.controllerDefined);
					}
					dss.ajaxAssyncRequestWithoutAnswer('/json/event/unsubscribe', {
						subscriptionID : iSubscriptID,
						name : 'heating-controller.answer'
					});
				});

			},
			notUsedWeek:null,
			loadTimedSettings : function() {
				var sString = dss.ajaxSyncRequest('/json/property/query', {
					'query' : '/scripts/heating-controller/time-profile/*(*)/*(*)'
				});
				this.timeValues = [];
				this.timeNightValues = [];
				this.notUsedWeek ={};
				var oValues = {};
				try {
					var result = Ext.JSON.decode(sString);

					if (result.ok) {
						var res = result.result['time-profile'];
						for ( var iIndex = 0; iIndex < res.length; iIndex++) {
							if  (res[iIndex].disabled!=true) {
								var iTarget; // 0=apartment, -1 night, alles
												// andere zone
								if (res[iIndex].target == 0) {
									iTarget = 0;
									if (res[iIndex].desiredMode != null)
										if (res[iIndex].desiredMode == OperationModes.Night)
											iTarget = -1;
									if (res[iIndex].resetingMode != null)
										if (res[iIndex].resetingMode == OperationModes.Night)
											iTarget = -1;
								} else
									iTarget = res[iIndex].target;
	
								var iOffset = 0;
								var day = "";
								for ( var kIndex = 0; kIndex < res[iIndex][res[iIndex].id].length; kIndex++) {
									var oObject = res[iIndex][res[iIndex].id][kIndex];
									if (oObject.offset != null)
										iOffset = oObject.offset;
									if (oObject.recurrenceOffset != null)
										day = oObject.recurrenceOffset;
								}
	
								if (iTarget == -1) {
										
									if (res[iIndex].resetingMode != null) {
										if (iOffset<12*60*60) {
											if (day == "MO")
												day = "SU";
											else if (day == "TU")
												day = "MO";
											else if (day == "WE")
												day = "TU";
											else if (day == "TH")
												day = "WE";
											else if (day == "FR")
												day = "TH";
											else if (day == "SA")
												day = "FR";
											else if (day == "SU")
												day = "SA";
										}
										if (this.timeNightValues[day] == null)
											this.timeNightValues[day] = {};
										this.timeNightValues[day].to = iOffset;
										this.timeNightValues[day].toID = res[iIndex].id;
									} else {
										if (iOffset<12*60*60) {
											if (day == "MO")
												day = "SU";
											else if (day == "TU")
												day = "MO";
											else if (day == "WE")
												day = "TU";
											else if (day == "TH")
												day = "WE";
											else if (day == "FR")
												day = "TH";
											else if (day == "SA")
												day = "FR";
											else if (day == "SU")
												day = "SA";
										}

										
										if (this.timeNightValues[day] == null)
											this.timeNightValues[day] = {};
										this.timeNightValues[day].from = iOffset;
										this.timeNightValues[day].fromID = res[iIndex].id;
									}
	
								} else if (iTarget > 0) {									

									
									if (this.notUsedWeek[iTarget]==null)
										this.notUsedWeek[iTarget]={};
									if (this.notUsedWeek[iTarget][day]==null)
										this.notUsedWeek[iTarget][day]={};

									if (res[iIndex].resetingMode != null) {
										this.notUsedWeek[iTarget][day].to=iOffset;
										this.notUsedWeek[iTarget][day].toID=res[iIndex].id;
									} else {
										this.notUsedWeek[iTarget][day].from=iOffset;
										this.notUsedWeek[iTarget][day].fromID=res[iIndex].id;
									}

									
									if (oValues['' + iTarget] == null)
										oValues['' + iTarget] = [];
									switch (day) {
									case 'MO':
										iOffset += (0 * 24 * 60 * 60);
										break;
									case 'TU':
										iOffset += (1 * 24 * 60 * 60);
										break;
									case 'WE':
										iOffset += (2 * 24 * 60 * 60);
										break;
									case 'TH':
										iOffset += (3 * 24 * 60 * 60);
										break;
									case 'FR':
										iOffset += (4 * 24 * 60 * 60);
										break;
									case 'SA':
										iOffset += (5 * 24 * 60 * 60);
										break;
									case 'SU':
										iOffset += (6 * 24 * 60 * 60);
										break;
									}
									if (iTarget != 0)
										if (res[iIndex].resetingMode != null)
											if (res[iIndex].resetingMode == OperationModes.NotUsed)
												res[iIndex].desiredMode = OperationModes.transparent;
									
									
										
									oValues['' + iTarget].push({
										time : iOffset,
										mode : res[iIndex].desiredMode,
										id : res[iIndex].id
									});
									/*
									 * { duration : 21600, type : 2 },
									 */
								}
							}
						}
					}

				} catch (e) {

				}
				/*
				 * if (oValues['0'] == 0) oValues['0'] = [ { time : 0, mode :
				 * OperationModes.Eco, id : null } ];
				 */
				for ( var sKey in oValues) {
					var oValue = oValues[sKey];
					oValue.sort(function(a, b) {
						return a.time - b.time;
					});
					var timeIndex = 0;
					var val = [];
					if (oValue.length > 0) {
						if (oValue[0].time != 0) {
							var oObject = {
								duration : oValue[0].time,
								type : oValue[oValue.length - 1].mode,
								id : oValue[oValue.length - 1].id
							};
							val.push(oObject);
						}
						timeIndex = oValue[0].time;
						for ( var iIndex = 0; iIndex < oValue.length - 1; iIndex++) {
							var oObject = {
								duration : (oValue[iIndex + 1].time - timeIndex),
								type : oValue[iIndex].mode,
								id : oValue[iIndex].id
							};
							val.push(oObject);
							timeIndex = oValue[iIndex + 1].time;
						}

						var oObject = {
							duration : ((7 * 24 * 60 * 60) - timeIndex),
							type : oValue[oValue.length - 1].mode,
							id : oValue[oValue.length - 1].id
						};
						val.push(oObject);
					}
					var zoneID = parseInt(sKey, 10);
					var oValueObjectForChart = {
						name : (zoneID == 0) ? _('Apartment') : dss.staticDataModel.getNameForZone(zoneID),
						id : zoneID,
						val : val
					};
					this.timeValues.push(oValueObjectForChart);
				}
                /*
				 * if no time profile for a usable zone found add an empty one
				 * in the graph
				 */
				for ( var iIndex = 0; iIndex < this.usableZones.length; iIndex++) {
					var oZoneObject = this.usableZones[iIndex];
					var fFound = false;
					for ( var jIndex = 0; jIndex < this.timeValues.length; jIndex++) {
						if (this.timeValues[jIndex].id == oZoneObject.ZoneID)
							fFound = true;
					}
					if (!fFound) {
						this.timeValues.push({
							name : oZoneObject.name,
							id : oZoneObject.ZoneID,
							val : [ {
								duration : 604800,
								type : -1
							} ]
						});
                    }
				}
                /* remove time profile for a not usable zone */
                for ( var jIndex = 0; jIndex < this.timeValues.length; jIndex++) {
					var fFound = false;
    				for ( var iIndex = 0; iIndex < this.usableZones.length; iIndex++) {
    					var oZoneObject = this.usableZones[iIndex];
						if (this.timeValues[jIndex].id == oZoneObject.ZoneID)
							fFound = true;
					}
					if (!fFound) {
                        this.removeTimeline(this.timeValues[jIndex].id);
						this.timeValues.splice(jIndex,1);
                    }
				}
                
			},
			waitAndReloadTimedSettings : function() {
				var me = this;
				var iSubscriptID = Math.floor(Math.random() * 100000000000);
				dss.ajaxSyncRequest('/json/event/subscribe', {
					subscriptionID : iSubscriptID,
					name : 'heating-controller.answer'
				});
				dss.ajaxAssyncRequestWithoutAnswer('/json/event/get', {
					subscriptionID : iSubscriptID,
					timeout : 60 * 1000
				}, function() {
                    me.waitAndReloadStatus(me.overviewPanel);
					me.loadTimedSettings();
					dss.ajaxAssyncRequestWithoutAnswer('/json/event/unsubscribe', {
						subscriptionID : iSubscriptID,
						name : 'heating-controller.answer'
					});
					me.updateTimeStore();
				});
			},
			loadGeneralSettings : function() {
				this.generalSettingsControl.radioSingleMultihouse.suspendEvents();
				this.generalSettingsControl.ckbCentralHeatingNightmode.suspendEvents();
				this.generalSettingsControl.radioHeatingAppGeneralSwitch.suspendEvents();
				/*
				 * this.generalSettingsControl.valveProtectionTime.suspendEvents();
				 * this.generalSettingsControl.valveProtectionStart.suspendEvents();
				 * this.generalSettingsControl.valveProtectionInterval.suspendEvents();
				 */
				this.generalSettingsControl.preHeatingHoliday.suspendEvents();
				this.generalSettingsControl.notUsedWhenAbsend.suspendEvents();
				var sString = dss.ajaxSyncRequest('/json/property/query2', {
					'query' : '/scripts/heating-controller/generalSettings(*)'
				});
				try {
					var result = Ext.JSON.decode(sString);
					if (result.ok) {
						if (result.result != null)
							if (result.result.generalSettings != null) {
								if (result.result.generalSettings.apartmentType != null) {
									this.generalSettingsControl.radioSingleMultihouse.setValue({
										singleApartment : result.result.generalSettings.apartmentType
									});
								}
								if (result.result.generalSettings.centralHeatingNightModeActive != null) {
									this.generalSettingsControl.ckbCentralHeatingNightmode.setValue(result.result.generalSettings.centralHeatingNightModeActive);
								}
								if (result.result.generalSettings.heatingEnabled != null) {
									this.generalSettingsControl.radioHeatingAppGeneralSwitch.setValue({
										centralSwitch : result.result.generalSettings.heatingEnabled
									});
								} else {
                                    dss.raiseEvent('heating-controller.general-settings', {
                                        actions : 'setHeatingEnabled',
                                        value : true
                                    });
                                    this.generalSettingsControl.radioHeatingAppGeneralSwitch.setValue({
										centralSwitch : true
									});
                                }
								if (result.result.generalSettings.useNotUsedOnAbsent != null) {
									this.generalSettingsControl.notUsedWhenAbsend.setValue(result.result.generalSettings.useNotUsedOnAbsent);
								}
								if (result.result.generalSettings.preheatingTime != null) {
									this.generalSettingsControl.preHeatingHoliday.setValue(result.result.generalSettings.preheatingTime);
								}
								if ((result.result.generalSettings.apartmentType != null) && (result.result.generalSettings.heatingEnabled != null)) {
									this.generalSettingsMissing = false;
								}
							}
					}

				} catch (e) {

				}
				this.generalSettingsControl.radioSingleMultihouse.resumeEvents();
				this.generalSettingsControl.ckbCentralHeatingNightmode.resumeEvents();
				this.generalSettingsControl.radioHeatingAppGeneralSwitch.resumeEvents();
				this.generalSettingsControl.preHeatingHoliday.resumeEvents();
				this.generalSettingsControl.notUsedWhenAbsend.resumeEvents();

				// this.temperaturePanel.columns[6].setVisible(!(this.generalSettingsControl.ckbCentralHeatingNightmode.getValue()));
				// this.newNightButton.setDisabled(this.generalSettingsControl.ckbCentralHeatingNightmode.getValue());

			},
			loadRoomParameter : function(grid) {
                var loadError = false;
				/*
				 * {name : 'zoneID}, {name : 'baseMode'}, {name :
				 * 'temperature'}, {name : 'lastTS'}, {name : 'sensorID'}, {name :
				 * 'actors'}, {name : 'states'}, {name : 'flags'}
				 */
				var sString = dss.ajaxSyncRequest('/json/property/query2', {
					'query' : '/scripts/heating-controller/controllerDefinitions/*(*)'
				});

				try {
					var result = Ext.JSON.decode(sString);
					this.heatingStore_controllerMode.removeAll();
					this.heatingStore_controllerMode.add({
						id : '0',
						name : _('no controller active'),
						mode : 0,
						object : {
							controllerMode : ControlModes.cControlModeOff
						}
					});
					for ( var sIndex in result.result) {
						var oObject = result.result[sIndex];
						this.heatingStore_controllerMode.add({
							id : '' + oObject.id,
							name : oObject.name,
							mode : parseInt('' + oObject.controllerMode),
							object : oObject
						});
					}
					this.heatingStore_controllerMode.add({
						id : '-2',
						name : _('default zone follower'),
						mode : 0,
						object : {
							controllerMode : ControlModes.cControlModeZoneFollower
						}
					});
					this.heatingStore_controllerMode.add({
						id : '-3',
						name : _('default fixed value'),
						mode : 0,
						object : {
							controllerMode : ControlModes.cControlModeFixedValue
						}
					});
					this.heatingStore_controllerMode.add({
						id : '-5',
						name : _('automatic mode'),
						mode : 0,
						object : {
							controllerMode : ControlModes.cControlModeAutomatic
						}
					});

				} catch (e) {

				}

				this.controllerDefined = false;
				var allreadyDefinedController = {};
				var allreadyDefinedEmergeny = {};
				var alreadyDefinedAutoselect = {};
                this.autoSelectMessage = [];
				var sString = dss.ajaxSyncRequest('/json/property/query2', {
					'query' : '/scripts/heating-controller/controllerConfig/*(*)'
				});
				try {
					var result = Ext.JSON.decode(sString);
					for ( var sIndex in result.result) {
						var oObject = result.result[sIndex];
						allreadyDefinedController[oObject.zoneID] = parseInt("" + oObject.definitionID, 10);
						if (oObject.emergenyValue != null)
							allreadyDefinedEmergeny[oObject.zoneID] = parseInt("" + oObject.emergenyValue, 10);
						if (oObject.autoselect != null) {
							alreadyDefinedAutoselect[oObject.zoneID] = oObject.autoselect;
						}
					}
				} catch (e) {

				}

				var oRArray = [];
				var sString = dss.ajaxSyncRequest('/json/property/query2', {
					'query' : '/apartment/dSMeters/*(dSUID)/*(temperature-control)/*(ZoneID)'
				});
				try {
					var zonesToBeShown=[];
					var result = Ext.JSON.decode(sString);
					for ( var sKey in result.result) {						
						oDSM = result.result[sKey];
						if (oDSM.capabilities['temperature-control'] != null)
							if (oDSM.capabilities['temperature-control']) {
								for (aKey in oDSM.zones) {
									var fFound = false;
									aKey = parseInt('' + aKey, 10);
									for ( var iIndex = 0; iIndex < zonesToBeShown.length; iIndex++) {
										if ((""+zonesToBeShown[iIndex]) == (""+aKey))
											fFound = true;
									}
									if (!fFound) {
										zonesToBeShown.push(aKey);										
									}
								}
							}
					}	

					for (var aKey in this.zoneControlValues) {
						if (this.zoneControlValues[aKey].IsConfigured) {
							var fFound=false;
							for (var jIndex=0;jIndex<zonesToBeShown.length;jIndex++){
								if ((""+aKey)==(""+zonesToBeShown[jIndex]))
									fFound=true;
							}
							if (!fFound) {
								zonesToBeShown.push(this.zoneControlValues[aKey].id);
							}
						}
                    }
					for (var kIndex=0;kIndex<zonesToBeShown.length;kIndex++) {	
						aKey=zonesToBeShown[kIndex];
						
						var fFound=false;
						for (var rIndex=0;rIndex<oRArray.length;rIndex++) {
							if (oRArray[rIndex].zoneID==aKey)
								fFound=true;
						}
						if (!fFound) {
						
							var sensorID = null;
							if (this.zoneControlValues[aKey] != null) {
								if (this.zoneControlValues[aKey].sensors != null) {
									for ( var jIndex = 0; jIndex < this.zoneControlValues[aKey].sensors.length; jIndex++) {
										if (this.zoneControlValues[aKey].sensors[jIndex].sensorType == 9)
											sensorID = this.zoneControlValues[aKey].sensors[jIndex].dsuid;
									}
								}
							}
	
							var oObject = {
								zoneID : aKey,
                                zoneName : dss.staticDataModel.getNameForZone(aKey),
								sensorID : sensorID,
								actors : [],
								parameter : null,
								baseMode : 0
							};
	
							var oDevices = dss.staticDataModel.getDevicesForZone(aKey, 48);
							for ( var iIndex = 0; iIndex < oDevices.length; iIndex++) {
								if (oDevices[iIndex].outputMode > 0)
									oObject.actors.push(oDevices[iIndex]);
							}
	
							if (this.zoneControlValues[aKey] != null) {
								if (alreadyDefinedAutoselect[aKey]) {
									oObject.baseMode = -5;
									if (allreadyDefinedController[aKey] != null) {
                                        var msg ='';
                                        switch (allreadyDefinedController[aKey]) {
                                            case AutomaticModeError.AutoUnknown:
                                            case AutomaticModeError.AutoOff: //intend fall through
                                                oObject.parameter = 0;                        
                                                msg = Ext.String.format2(_('For room {zonename} no operation mode could be defined'), {
                                                    zonename : dss.staticDataModel.getNameForZone(aKey)}); 
                                                break;
                                            case AutomaticModeError.AutoNothingSelected:
                        						oObject.baseMode = -6;
                                                oObject.parameter = 0;                        
                                                msg = Ext.String.format2(_('No controller mode for room {zonename} could automatically be selected. There is no sensor in this room'), {
                                                    zonename : dss.staticDataModel.getNameForZone(aKey)});
                                                break;
                                            case AutomaticModeError.AutoSelectionFailed:
                                                oObject.baseMode = -6;
                                                oObject.parameter = 0;
                                                msg = Ext.String.format2(_('No controller mode for room {zonename} could automatically be selected. Please choose a controller mode.'), {
                                                    zonename : dss.staticDataModel.getNameForZone(aKey)});
                                                break;
                                            case AutomaticModeError.AutoSelectionNotPossible:
    											oObject.baseMode = -7;
                                                oObject.parameter = 0;
                                                msg = Ext.String.format2(_('No controller mode for room  {zonename} could automatically be selected. Please select the native device in the property of the device'), {
                                                    zonename : dss.staticDataModel.getNameForZone(aKey)});
                                                break;
                                            default:
                                                oObject.parameter = allreadyDefinedController[aKey];
                                                break;
                                        }
                                        if (msg != '') {
                                            this.autoSelectMessage.push(msg);
                                        }
                                    }
								}
								else if (this.zoneControlValues[aKey].ControlMode != null) {
									if (this.zoneControlValues[aKey].ControlMode == 1) {
										oObject.baseMode = -1;
										if (allreadyDefinedController[aKey] != null) {
											if (allreadyDefinedController[aKey] > 0) {
												oObject.baseMode = allreadyDefinedController[aKey];
											}
										}
									}
									if (this.zoneControlValues[aKey].ControlMode == 2)
										oObject.baseMode = -2;
									if (this.zoneControlValues[aKey].ControlMode == 3)
										oObject.baseMode = -3;
									if (this.zoneControlValues[aKey].ControlMode == 4)
										oObject.baseMode = -4;
								}
							}
							if (allreadyDefinedEmergeny[aKey] != null) {
								oObject.emergenyValue = allreadyDefinedEmergeny[aKey];
                            } else {
                                oObject.emergenyValue = 75;
                            }
							if (oObject.baseMode == -2) {							
								oObject.referenceZone = this.zoneControlValues[aKey].config.ReferenceZone;
								oObject.ctrlOffset = this.zoneControlValues[aKey].config.CtrlOffset;                            
							}
							if (oObject.baseMode != 0) {
								this.controllerDefined = true;
							}
                            if ((sensorID != null) || (oObject.actors.length > 0) || (oObject.baseMode != 0 && oObject.baseMode != -5)) {
                                oRArray.push(oObject);
                            }
						}
					}
				} catch (e) {

				}
				if (grid != null) {
					this.roomGrid = grid;
                    grid.editorWindow.editControls['referenceZone'].cboCombo.getStore().removeAll();
                    for ( var iIndex = 0; iIndex < this.usableZones.length; iIndex++) {
                        var oObject = this.usableZones[iIndex];
                        if (oObject.ZoneID != 0) {
                            grid.editorWindow.editControls['referenceZone'].cboCombo.getStore().add({
                                id : '' + oObject.ZoneID,
                                name : oObject.name
                            });
                        }
                    }
                }
				this.roomGrid.loadParameter(oRArray);
				this.loadGeneralSettings();
				if (HeatingControl != undefined) {
					tabActivation(!this.controllerDefined);
				}
				if (this.waitDialog != null) {
					this.waitDialog.hide();
					this.waitDialog = null;
				}
                if (grid) {
                    if (loadError) {
                        grid.setStatus(_('Error'));
                    } else {
                        grid.setStatus(_('Ready'));
                    }
                }
			},
			waitAndReloadRoomParameter : function() {
				var me = this;
				var iSubscriptID = Math.floor(Math.random() * 100000000000);
				dss.ajaxSyncRequest('/json/event/subscribe', {
					subscriptionID : iSubscriptID,
					name : 'heating-controller.answer'
				});
				dss.ajaxAssyncRequestWithoutAnswer('/json/event/get', {
					subscriptionID : iSubscriptID,
					timeout : 60 * 1000
				}, function() {
					this.temperatureControlValues = null;
					me.fetchZoneControlValues(true);
					me.loadRoomParameter();
					me.temperaturePanel.fireEvent('loadDataStore', me.temperaturePanel);
					me.loadTimedSettings();
					me.updateTimeStore();
					me.loadStatusPanel();
					dss.ajaxAssyncRequestWithoutAnswer('/json/event/unsubscribe', {
						subscriptionID : iSubscriptID,
						name : 'heating-controller.answer'
					});
				});

			},
			fetchZoneControlValues : function(fReload) {
                var success = true;
                this.setErrorState(false);
				if (fReload == null)
					if ((this.zoneControlValues != null) && (this.temperatureControlValues != null))
						return success;
				this.zoneControlValues = {};

				var sZoneControlStatus = dss.ajaxSyncRequest('/json/apartment/getTemperatureControlStatus', {});
				resultControlStatus = Ext.JSON.decode(sZoneControlStatus);
                if (!resultControlStatus.ok) {
                    console.log('failure in JSON request apartment/getTemperatureControlStatus');
                    success = false;
                    this.setErrorState(true, 'Error getting current status of temperature control for all zones');
                }
				var oZones = dss.staticDataModel.getZoneArray();
				for ( var iIndex = 0; iIndex < oZones.length; iIndex++) {
					zoneID = oZones[iIndex].ZoneID;
					if (zoneID !== 0) { // all rooms
						for ( var jIndex = 0; jIndex < resultControlStatus.result.zones.length; jIndex++)
							if (resultControlStatus.result.zones[jIndex].id == zoneID)
								this.zoneControlValues[zoneID] = resultControlStatus.result.zones[jIndex];
					}
				}

				var sStringParameter = dss.ajaxSyncRequest('/json/apartment/getTemperatureControlConfig', {});
				var resultParameter = Ext.JSON.decode(sStringParameter);
        
				if (resultParameter.ok) {
					for (var iIndex=0;iIndex<resultParameter.result.zones.length;iIndex++) {
						this.zoneControlValues[resultParameter.result.zones[iIndex].id].config=resultParameter.result.zones[iIndex];
					}
                } else {
                    loadError = true;
                    this.setErrorState(true, 'Error getting the temperature configuration control for all zones');
                }
				
				
				
				
				var sZoneSensorStatus = dss.ajaxSyncRequest('/json/apartment/getAssignedSensors', {});
				resultSensorStatus = Ext.JSON.decode(sZoneSensorStatus);
                if (!resultSensorStatus.ok) {
                    console.log('failure in JSON request apartment/getAssignedSensors');
                    success = false;
                    this.setErrorState(true, 'Error getting list of assigned sensors for all zones');
                }

				for ( var iIndex = 0; iIndex < resultSensorStatus.result.zones.length; iIndex++) {
					var zoneID = resultSensorStatus.result.zones[iIndex].id;
					if (this.zoneControlValues[zoneID] != null)
						this.zoneControlValues[zoneID].sensors = resultSensorStatus.result.zones[iIndex].sensors;
				}
				this.usableZones = [];
				this.temperatureControlValues = {};
				
				var resultTemperature = Ext.JSON.decode(dss.ajaxSyncRequest('/json/apartment/getTemperatureControlValues', {}));
				
				for ( var sKey in this.zoneControlValues) {
					if ((this.zoneControlValues[sKey].ControlMode == 1) || (this.zoneControlValues[sKey].ControlMode == 3)) {
						for ( var iIndex = 0; iIndex < dss.staticDataModel.getZoneArray().length; iIndex++) {
							if (dss.staticDataModel.getZoneArray()[iIndex].ZoneID == parseInt(sKey, 10))
								this.usableZones.push(dss.staticDataModel.getZoneArray()[iIndex]);
						}

                        if (resultTemperature.ok) { 
                        	for (var iIndex=0;iIndex<resultTemperature.result.zones.length;iIndex++) {
                        		if (resultTemperature.result.zones[iIndex].id==sKey)
                        			this.temperatureControlValues[sKey] = resultTemperature.result.zones[iIndex];
                        	}                        	                            
                        } else {
                            console.log('failure in JSON request zone/getTemperatureControlValues');
                            success = false;
                            this.setErrorState(true, 'Error getting the temperature control operation mode preset values for all zones');
                        }
					}
				}
				
				var resultSensorTemperature = Ext.JSON.decode(dss.ajaxSyncRequest('/json/apartment/getSensorValues', {}));				

				if (resultSensorTemperature.ok) {
					for (var iIndex=0;iIndex<resultSensorTemperature.result.zones.length;iIndex++) {
						for (var jIndex=0;jIndex<resultSensorTemperature.result.zones[iIndex].values.length;jIndex++) {
							if (resultSensorTemperature.result.zones[iIndex].values[jIndex].TemperatureValue!=null)
								this.zoneControlValues[resultSensorTemperature.result.zones[iIndex].id].actualTemperature=resultSensorTemperature.result.zones[iIndex].values[jIndex].TemperatureValue;							
						}
					}
				}				
				
				if (Ext.data.StoreManager.lookup('usableZones') == null) {
					Ext.create('Ext.data.Store', {
						fields : [ 'id', 'name' ],
						storeId : 'usableZones',
						proxy : {
							type : 'memory',
							reader : {
								type : 'json',
								root : 'items'
							}
						}
					});
				}

				var newValue = {};
				for ( var iIndex = 0; iIndex < this.usableZones.length; iIndex++) {
					newValue['id'] = "" + this.usableZones[iIndex].ZoneID;
					newValue['name'] = "" + this.usableZones[iIndex].name;
					Ext.data.StoreManager.lookup('usableZones').add([ newValue ]);
				}
                /*
				 * iterate over all dSMs an get zones with temperature control
				 * capability
				 */
                this.possibleZones = [];
                var sString = dss.ajaxSyncRequest('/json/property/query2', {
					'query' : '/apartment/dSMeters/*(dSUID)/*(temperature-control)/*(ZoneID)'
				});
                var result = Ext.JSON.decode(sString);
                for ( var sKey in result.result) {						
                    var oDSM = result.result[sKey];
                    if (oDSM.capabilities['temperature-control'] != null)
                        if (oDSM.capabilities['temperature-control']) {
                            for (aKey in oDSM.zones) {
                                var fFound = false;
                                aKey = parseInt('' + aKey, 10);
                                for ( var iIndex = 0; iIndex < this.possibleZones.length; iIndex++) {
                                    if ((""+this.possibleZones[iIndex]) == (""+aKey))
                                        fFound = true;
                                }
                                if (!fFound) {
                                    this.possibleZones.push(aKey);										
                                }
                            }
                        }
                }	
                
                /* now take apartment zone control values and add missing zones */
                for (var aKey in this.zoneControlValues) 
                    if (this.zoneControlValues[aKey].IsConfigured) {
                        var fFound=false;
                        for (var jIndex=0;jIndex<this.possibleZones.length;jIndex++){
                            if ((""+aKey)==(""+this.possibleZones[jIndex]))
                                fFound=true;
                        }
                        if (!fFound) {
                            this.possibleZones.push(this.zoneControlValues[aKey].id);
                        }
                    }

                if (this.heatingStore_referenceZone == null) {
					this.heatingStore_referenceZone = Ext.create('Ext.data.Store', {
						storeId : 'heatingStore_referenceZone',
						fields : [ 'id', 'name' ],
						proxy : {
							type : 'memory',
							reader : {
								type : 'json',
								root : 'items'
							}
						}
					});
				}
                this.heatingStore_referenceZone.removeAll();
				for ( var iIndex = 0; iIndex < this.usableZones.length; iIndex++) {
					var oObject = this.usableZones[iIndex];
					if (oObject.ZoneID != 0) {
						this.heatingStore_referenceZone.add({
							id : '' + oObject.ZoneID,
							name : oObject.name
						});
					}
				}
                
                return success;
			},
			loadTemperatureParameter : function(grid) {
				var success = this.fetchZoneControlValues();

				var zoneID;
				var resultControlStatus, resultControlValues;
				var oResultObject = [];
                if (this.temperatureControlValues) {
                    for ( var sKey in this.temperatureControlValues) {
                        zoneID = sKey;
                        resultControlStatus = this.zoneControlValues[zoneID];
                        resultControlValues = this.temperatureControlValues[zoneID];
                        zoneObject = {
                            zoneID : zoneID,
                            zoneName : dss.staticDataModel.getNameForZone(zoneID),
                            temperature : (resultControlStatus.TemperatureValue) ? resultControlStatus.TemperatureValue : null,
                            effectiveNominalTemperature : (resultControlStatus) ? resultControlStatus.NominalValue : null,
                            operationMode : (resultControlStatus) ? resultControlStatus.OperationMode : null,
                            nominalTemperatureComfort : ((resultControlValues) && (resultControlValues.Comfort > -43)) ? resultControlValues.Comfort : null,
                            nominalTemperatureEco : ((resultControlValues) && (resultControlValues.Economy > -43)) ? resultControlValues.Economy : null,
                            nominalTemperatureNotUsed : ((resultControlValues) && (resultControlValues.NotUsed > -43)) ? resultControlValues.NotUsed : null,
                            nominalTemperatureNight : ((resultControlValues) && (resultControlValues.Night > -43)) ? resultControlValues.Night : null,
                            nominalTemperatureHoliday : ((resultControlValues) && (resultControlValues.Holiday > -43)) ? resultControlValues.Holiday : null,
                            activeMode : resultControlValues.IsConfigured && ((resultControlStatus.ControlMode == 1) || (resultControlStatus.ControlMode == 3)),
                            inactiveMode : (resultControlValues) ? ((!resultControlValues.IsConfigured) || (resultControlStatus.ControlMode == 0) || (resultControlStatus.ControlMode == 4)
                                    || (resultControlStatus.ControlMode == 2)) : null, // no

                            controlMode : (resultControlStatus) ? resultControlStatus.ControlMode : null
                        };
                        oResultObject.push(zoneObject);
                    }
                }
				grid.loadParameter(oResultObject);
                return success;
			},
			reloadTemperatureSettings : function(grid, values) {
				this.temperatureControlValues = null;
				this.loadTemperatureParameter(grid);
			},
			saveTemperaturParameter : function(values) {
				var saveObject = {
					'id' : values.zoneID,
					'Comfort' : values.nominalTemperatureComfort,
					'Economy' : values.nominalTemperatureEco,
					'NotUsed' : values.nominalTemperatureNotUsed,
					'Night' : values.nominalTemperatureNight,
					'Holiday' : values.nominalTemperatureHoliday
				};
				var sResult = Ext.JSON.decode(dss.ajaxSyncRequest('/json/zone/setTemperatureControlValues', saveObject));
				if (!sResult.ok) {
					console.log('failure during save nominal temperature values');
				}

				sResult = Ext.JSON.decode(dss.ajaxSyncRequest('/json/zone/getTemperatureControlStatus', {id:values.zoneID}));
				if (sResult.ok) {
					var sceneNumber=sResult.result.OperationMode;
					dss.ajaxSyncRequest('/json/zone/callScene', {id:values.zoneID,groupID:48,sceneNumber:sceneNumber});
				}
				
			}
		});

Ext.onReady(function() {
	initializeConsole();
	dss.buildUpLang([ 'locale/{languageSuffix}/heating.po' ]);

	dss.raiseEvent('heating-controller.cloud', {
		actions: 'sync'
	});

	HeatingControl = Ext.create('DSS.addon.HeatingControl');
	tabActivation(!HeatingControl.controllerDefined);
	HeatingControl.initPage();
});
